Package: countsimQC
Type: Package
Title: Compare Characteristic Features of Count Data Sets
Version: 1.31.0
Authors@R: c(person("Charlotte", "Soneson", role = c("aut", "cre"), 
    email = "charlottesoneson@gmail.com", comment = c(ORCID = "0000-0003-3833-2169")))
Description: countsimQC provides functionality to create a comprehensive report 
    comparing a broad range of characteristics across a collection of count 
    matrices. One important use case is the comparison of one or more synthetic 
    count matrices to a real count matrix, possibly the one underlying the 
    simulations. However, any collection of count matrices can be compared.
License: GPL (>=2)
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: rmarkdown (>= 2.5), edgeR, DESeq2 (>= 1.16.0), dplyr, tidyr,
        ggplot2, grDevices, tools, SummarizedExperiment, genefilter,
        DT, GenomeInfoDbData, caTools, randtests, stats, utils,
        methods, ragg, rlang
RoxygenNote: 7.3.3
Suggests: knitr, testthat
VignetteBuilder: knitr
biocViews: Microbiome, RNASeq, SingleCell, ExperimentalDesign,
        QualityControl, ReportWriting, Visualization, ImmunoOncology
URL: https://github.com/csoneson/countsimQC
BugReports: https://github.com/csoneson/countsimQC/issues
git_url: https://git.bioconductor.org/packages/countsimQC
git_branch: devel
git_last_commit: 7740a95
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:03:05 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>)
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>

# Generated by roxygen2: do not edit by hand

export(countsimQCReport)
export(generateIndividualPlots)
import(DESeq2)
import(DT)
import(GenomeInfoDbData)
import(SummarizedExperiment)
import(dplyr)
import(edgeR)
import(genefilter)
import(ggplot2)
import(ragg)
import(tidyr)
importFrom(caTools,trapz)
importFrom(edgeR,cpm)
importFrom(genefilter,rowVars)
importFrom(ggplot2,is_ggplot)
importFrom(grDevices,n2mfrow)
importFrom(methods,is)
importFrom(randtests,runs.test)
importFrom(rlang,.data)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_exec)
importFrom(rmarkdown,render)
importFrom(stats,chisq.test)
importFrom(stats,cor)
importFrom(stats,ecdf)
importFrom(stats,ks.test)
importFrom(stats,model.matrix)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,combn)

# countsimQC
[![R build status](https://github.com/csoneson/countsimQC/workflows/R-CMD-check/badge.svg)](https://github.com/csoneson/countsimQC/actions)

`countsimQC` is an R package that provides functionality to create a 
comprehensive report comparing many different characteristics across multiple 
count data sets. One important use case is comparing one or more 
synthetic (e.g., RNA-seq) count matrices to a real count matrix, possibly the 
one based on which the synthetic data sets were generated. However, any 
collection of one or more count matrices can be visualized and compared.

If you use `countsimQC` for your work, we appreciate if you cite the 
accompanying paper:

- Soneson C and Robinson MD: [Towards unified quality verification of synthetic count data with countsimQC](https://academic.oup.com/bioinformatics/article/doi/10.1093/bioinformatics/btx631/4345646/Towards-unified-quality-verification-of-synthetic). Bioinformatics 34(4):691-692 (2018).

## Installation
`countsimQC` can be installed from 
[Bioconductor](https://www.bioconductor.org/packages/countsimQC/) with the 
following commands. Note that R version >= 3.5 and Bioconductor 
version >= 3.8 are required in order to use the `BiocManager` package. 
If you have an older version of R (3.4), you can still install 
`countsimQC` v0.5.4 (see the `Releases` tab in the GitHub repository). 
Please see the `NEWS` file for differences between versions.

```
## Install `BiocManager` if needed
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
    
## Install countsimQC
BiocManager::install("countsimQC")
```

## Getting started
To run `countsimQC` and generate a report, you simply need to call the
function `countsimQCReport()`, with an input consisting of a named list of
`DESeqDataSets` (see the
[DESeq2](https://www.bioconductor.org/packages/DESeq2/)
package for a description of this class). Each `DESeqDataSet` should
correspond to one data set and contain a count matrix, a data frame with sample
information and a design formula, which is needed for proper dispersion 
calculations. To generate a `DESeqDataSet` from a count matrix `counts`, a 
sample information data frame `sample_df` and a design formula `formula` 
(of the form `~ predictors`), you can do as follows:

```
library(DESeq2)
dds <- DESeqDataSetFromMatrix(countData = counts, 
                              colData = sample_df,
                              design = formula)
```
There are many other ways of generating valid `DESeqDataSets`, depending on in 
what form your counts are (e.g., reading directly from 
[HTSeq](http://htseq.readthedocs.io/en/release_0.9.1/) output, or from a [tximport](http://bioconductor.org/packages/release/bioc/html/tximport.html) 
output object (see the 
[DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html) [vignette](http://bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.html)). 
 
`countsimQC` contains an small example list with subsets of three data sets: 
two synthetic ones and the real data set that was used to generate them. The 
following code generates a comparative report for these three data sets:

```
library(countsimQC)
data(countsimExample)
countsimQCReport(ddsList = countsimExample, 
                 outputFile = "countsimReport.html", 
                 outputDir = "./", 
                 description = "This is a comparison of three count data sets.")
```

For more detailed information about how to use the package, we refer to the vignette:

```
browseVignettes("countsimQC")
```

## Example reports

- [Comparison of 16S microbiome species count matrices for four body subsites from the Human Microbiome Project](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/HMP_sampled_datasets_countsimQC.html)
- [Comparison of three real bulk RNA-seq data sets](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/bulkrnaseq_crossdataset_countsimQC.html)
- [Comparison of gene- and transcript-level count matrices for a single-cell RNA-seq data set](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/GSE74596_genevstx_countsimQC.html) 
- [Comparison of four real scRNA-seq data sets](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/scrnaseq_crossdataset_countsimQC.html)
- [Comparison of two simulated scRNA-seq data sets to the underlying real data set](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/GSE48968-GPL13112_2simulations_countsimQC.html)
- [Comparison of six simulated bulk RNA-seq data set with different number of genes](http://imlspenticton.uzh.ch/robinson_lab/countsimQC_example_reports/countsimQC_compcodeR_simulations.html)

#' Calculate dispersions
#'
#' Calculate the dispersions for each data set in a list of DESeqDataSets, using
#' both edgeR and DESeq2.
#'
#' @param ddsList A list of DESeqDataSets
#' @param maxNForDisp If any data set contains more than \code{maxNForDisp}
#'   samples, \code{maxNForDisp} of them will be randomly sampled before the
#'   dispersions are calculated, in order to speed up calculations
#'
#' @return A list of the same length as the input list. Each element in the list
#'   is itself a list, containing a DGEList and a DESeqDataSet with calculated
#'   dispersions.
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
#' @import edgeR DESeq2 SummarizedExperiment
#' @importFrom stats model.matrix
#'
calculateDispersionsddsList <- function(ddsList, maxNForDisp) {
  lapply(ddsList, function(ds) {
    ## If the number of samples exceeds maxNForDisp, select random subset of
    ## samples to use for dispersion calculations (do it here so that the same
    ## set of samples is used for edgeR and DESeq2)
    if (ncol(ds) > maxNForDisp) {
      keepSamples <- sample(seq_len(ncol(ds)), maxNForDisp, replace = FALSE)
    }

    ## --------------------------- edgeR --------------------------- ##
    ## Define DGEList
    dge <- edgeR::DGEList(counts = DESeq2::counts(ds))
    ## Calculate normalization factors
    dge <- edgeR::calcNormFactors(dge)
    if (ncol(dge) > maxNForDisp) {
      ## Subset DGEList if the number of samples exceeds maxNForDisp and
      ## estimate dispersions
      dgetmp <- dge[, keepSamples]
      destmp <- stats::model.matrix(
        DESeq2::design(ds),
        data = droplevels(SummarizedExperiment::colData(ds)[keepSamples, ,
                                                            drop = FALSE])
      )
      dgetmp <- edgeR::estimateDisp(dgetmp, design = destmp)

      ## Add dispersion estimates to original DGEList
      stopifnot(all(rownames(dge) == rownames(dgetmp)))
      dge$tagwise.dispersion <- dgetmp$tagwise.dispersion
      dge$common.dispersion <- dgetmp$common.dispersion
      dge$trended.dispersion <- dgetmp$trended.dispersion
      dge$prior.df <- dgetmp$prior.df
      dge$AveLogCPM <- edgeR::aveLogCPM(dge)
      dge$AveLogCPMDisp <- dgetmp$AveLogCPM
    } else {
      ## No subsampling, estimate dispersion from full DGEList
      des <- stats::model.matrix(DESeq2::design(ds),
                                 data = SummarizedExperiment::colData(ds))
      dge <- edgeR::estimateDisp(dge, design = des)
      dge$AveLogCPMDisp <- dge$AveLogCPM
    }
    ## --------------------------- DESeq2 -------------------------- ##
    ## Calculate size factors
    dds <- DESeq2::estimateSizeFactors(ds, type = "poscounts")
    if (ncol(dds) > maxNForDisp) {
      ## Subset DESeqDataSet if the number of samples exceeds maxNForDisp and
      ## estimate dispersions
      ddstmp <- dds[, keepSamples]
      colData(ddstmp) <- droplevels(colData(ddstmp))
      ddstmp <- DESeq2::estimateDispersions(ddstmp, quiet = TRUE)

      ## Add dispersion estimates to original DESeqDataSet
      stopifnot(all(rownames(dds) == rownames(ddstmp)))
      rowData(dds)$dispGeneEst <- rowData(ddstmp)$dispGeneEst
      rowData(dds)$dispFit <- rowData(ddstmp)$dispFit
      rowData(dds)$dispersion <- rowData(ddstmp)$dispersion
      rowData(dds)$baseMean <- rowMeans(counts(dds, normalized = TRUE))
      rowData(dds)$baseMeanDisp <- rowData(ddstmp)$baseMean
    } else {
      ## No subsampling, estimate dispersion from full DESeqDataSet
      dds <- DESeq2::estimateDispersions(dds, quiet = TRUE)
      rowData(dds)$baseMeanDisp <- rowData(dds)$baseMean
    }
    ## Check that the genes are in the same order in the DGEList and
    ## DESeqDataSet
    stopifnot(all(rownames(dge) == rownames(dds)))
    list(dge = dge, dds = dds)
  })
}

#' Calculate statistics for pairwise comparison of data sets
#'
#' Calculate a range of statistics and p-values for comparison of two data sets.
#'
#' @param df The input data frame. Must contain at least a column named
#'   'dataset' and an additional column with values to use as the basis for the
#'   comparison.
#' @param ds1,ds2 The names of the two data sets to be compared.
#' @param column The name of the column(s) of \code{df} to be used as the basis
#'   for the comparison.
#' @param subsampleSize The number of observations for which certain
#'   time-consuming statistics will be calculated. The observations will be
#'   selected randomly among the rows of \code{df}.
#' @param permute Whether to permute the dataset column of \code{df} before
#'   calculating the statistics.
#' @param kmin,kfrac For statistics that require the extraction of k nearest
#'   neighbors of a given point, the number of neighbors will be max(kmin, kfrac
#'   * nrow(df)).
#' @param xmin,xmax Smallest and largest value of \code{column}, used to
#'   normalize the x-axis when calculating the area between the eCDFs.
#'
#' @return A vector with statistics and p-values
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
#' @importFrom stats ks.test ecdf chisq.test
#' @importFrom rlang .data
#' @importFrom caTools trapz
#' @importFrom randtests runs.test
#'
calculateStats <- function(df, ds1, ds2, column, subsampleSize,
                           permute = FALSE, kmin, kfrac, xmin, xmax) {
  ## Remove rows with NA values in column(s) of interest
  df <- df[rowSums(is.na(df[, column, drop = FALSE])) == 0, ]

  ## Permute dataset column if requested
  if (permute) {
    df$dataset <- sample(df$dataset, nrow(df))
  }

  if (length(column) == 1) {
    ## Kolmogorov-Smirnov test
    kst <- stats::ks.test(df[, column][df$dataset == ds1],
                          df[, column][df$dataset == ds2])

    ## Area between ecdfs
    e1 <- stats::ecdf(df[, column][df$dataset == ds1])
    e2 <- stats::ecdf(df[, column][df$dataset == ds2])
    xv <- sort(unique(df[, column]))
    ediff <- abs(e1(xv) - e2(xv))
    earea <- caTools::trapz((xv - xmin)/(xmax - xmin), ediff)
  }

  ## Silhouette width and nearest neighbor distribution for subsample of
  ## observations
  ## Calculate overall proportion of values from the two data sets
  overallprop <- table(factor(df$dataset, levels = c(ds1, ds2)))
  overallprop <- overallprop/sum(overallprop)
  nrep <- 1
  chisilh <- t(vapply(seq_len(nrep), function(m) {
    ## Select subset of observations for calculation of silhouette widths and
    ## nearest neighbor distributions
    idx <- sample(seq_len(nrow(df)), min(nrow(df), subsampleSize))

    ## For each selected observation, calculate statistics
    tmp <- t(vapply(idx, function(j) {

      ## Distances to all observations
      dists <- sqrt(rowSums((sweep(df[, column, drop = FALSE],
                                   2, unlist(df[j, column,
                                                drop = FALSE]), "-")) ^ 2))
      dists_this <- dists[df$dataset == df$dataset[j]]
      dists_other <- dists[df$dataset != df$dataset[j]]

      ## Silhouette width
      silh <- (mean(dists_other, na.rm = TRUE) -
                 mean(dists_this, na.rm = TRUE)) /
        max(mean(dists_other, na.rm = TRUE), mean(dists_this, na.rm = TRUE))

      ## Local silhouette width
      dists_this_local <-
        sort(dists_this)[seq_len(max(kmin, kfrac * nrow(df)) *
                                   overallprop[df$dataset[j]])]
      dists_other_local <-
        sort(dists_other)[seq_len(max(kmin, kfrac * nrow(df)) *
                                    (1 - overallprop[df$dataset[j]]))]
      silh_local <-
        (mean(dists_other_local, na.rm = TRUE) -
           mean(dists_this_local, na.rm = TRUE)) /
        max(mean(dists_other_local, na.rm = TRUE),
            mean(dists_this_local, na.rm = TRUE))
      if (all(c(dists_this_local, dists_other_local) == 0)) {
        silh_local <- 0
      }

      ## Chi-square test comparing distribution of data set labels among
      ## nearest neighbors to the overall distribution
      x <- df$dataset[order(dists)][seq_len(max(kmin, kfrac * nrow(df)))]
      chisqp <- stats::chisq.test(x = table(factor(x, levels = c(ds1, ds2))),
                                  p = overallprop)$p.value
      ## Return values
      c(silh = silh, chisqp = chisqp, silh_local = silh_local)
    }, c(silh = NA_real_, chisqp = NA_real_, silh_local = NA_real_)))
    c(silh = mean(tmp[, "silh"], na.rm = TRUE),
      silhlocal = mean(tmp[, "silh_local"], na.rm = TRUE),
      chisqp = mean(tmp[, "chisqp"] <= 0.05, na.rm = TRUE))
  }, c(silh = NA_real_, silhlocal = NA_real_, chisqp = NA_real_)))

  ## Runs test
  if (length(column) == 1) {
    df <- df %>% arrange(.data[[column]])
    runs_res <- randtests::runs.test(as.numeric(df$dataset == ds1),
                                     threshold = 0.5,
                                     alternative = "left.sided", plot = FALSE)
  }

  if (length(column) == 1)
    c(ksstatistic = signif(kst$statistic, 3),
      kspvalue = signif(kst$p.value, 3),
      diffarea = signif(earea, 3),
      runsstatistic = signif(runs_res$statistic, 3),
      runspvalue = signif(runs_res$p.value, 3),
      NNmismatch = signif(mean(chisilh[, "chisqp"]), 3),
      avesilh = signif(mean(chisilh[, "silh"]), 3),
      avesilhlocal = signif(mean(chisilh[, "silhlocal"]), 3))
  else
    c(NNmismatch = signif(mean(chisilh[, "chisqp"]), 3),
      avesilh = signif(mean(chisilh[, "silh"]), 3),
      avesilhlocal = signif(mean(chisilh[, "silhlocal"]), 3))
}

#' Return a vector of NA scores
#'
#' @param n Number of columns to use for the comparison
#' @param withP Whether or not to include p-value columns
#'
#' @return A vector with NA values for all applicable statistics
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
defaultStats <- function(n, withP = FALSE) {
  if (n == 1) {
    if (withP) {
      c(ksstatistic = NA_real_, kspvalue = NA_real_, diffarea = NA_real_,
        runsstatistic = NA_real_, runspvalue = NA_real_, NNmismatch = NA_real_,
        avesilh = NA_real_, avesilhlocal = NA_real_, NNmismatchP = NA_real_,
        avesilhP = NA_real_, avesilhlocalP = NA_real_, diffareaP = NA_real_)
    } else {
      c(ksstatistic = NA_real_, kspvalue = NA_real_, diffarea = NA_real_,
        runsstatistic = NA_real_, runspvalue = NA_real_, NNmismatch = NA_real_,
        avesilh = NA_real_, avesilhlocal = NA_real_)
    }
  } else {
    if (withP) {
      c(NNmismatch = NA_real_, avesilh = NA_real_, avesilhlocal = NA_real_,
        NNmismatchP = NA_real_, avesilhP = NA_real_, avesilhlocalP = NA_real_)
    } else {
      c(NNmismatch = NA_real_, avesilh = NA_real_, avesilhlocal = NA_real_)
    }
  }
}

suppressPackageStartupMessages({
  library(MultiAssayExperiment)
  library(splatter)
  library(DESeq2)
})

mae <- readRDS(url("http://imlspenticton.uzh.ch/robinson_lab/conquer/data-mae/GSE74596.rds"))
config <- list(groupid = "source_name_ch1",
               keepgroups = c("Single_cell_RNA-seq_NKT0", "Single_cell_RNA-seq_NKT17")
)
mae <- updateObject(mae)
pdata <- colData(mae)
groupid <- config$groupid
keepgroups <- config$keepgroups

counts <- assays(experiments(mae)[["gene"]])[["count_lstpm"]]
stopifnot(all(colnames(counts) == rownames(pdata)))
keep <- which(pdata[, groupid] %in% keepgroups)
counts <- round(counts[, keep])
countsnz <- counts[rowSums(counts > 0) > 1, ]
countsz <- counts[rowSums(counts > 0) <= 1, ]
group <- as.character(pdata[keep, groupid])

dds_orig <- DESeqDataSetFromMatrix(
  countData = countsnz,
  colData = data.frame(group = group, sample = colnames(countsnz),
                       row.names = colnames(countsnz),
                       stringsAsFactors = FALSE),
  design = ~ group)

## splat
params <- splatEstimate(countsnz)
sim <- splatSimulate(params, method = "groups", group.prob = c(table(group))/length(group))
dds_splat <- DESeqDataSetFromMatrix(
  countData = round(counts(sim) * mean(colSums(counts(dds_orig)))/mean(colSums(counts(sim)))),
  colData = colData(sim),
  design = ~ Group)

## Lun
params <- lunEstimate(countsnz)
sim <- lunSimulate(params, groupCells = c(table(group)))
dds_lun <- DESeqDataSetFromMatrix(
  countData = round(counts(sim) * mean(colSums(counts(dds_orig)))/mean(colSums(counts(sim)))),
  colData = colData(sim),
  design = ~ Group)

countsimExample <- list(Original = dds_orig[1:10000, c(1:5, 84:89)],
                        Sim1 = dds_splat[1:10000, c(1:5, 84:89)],
                        Sim2 = dds_lun[1:10000, c(1:5, 84:89)])
devtools::use_data(countsimExample, pkg = "../..", overwrite = TRUE)

countsimExample_dfmat <- list(Original = dds_orig[1:10000, c(1:5, 84:89)],
                              Sim1 = as.matrix(counts(dds_splat[1:10000, c(1:5, 84:89)])),
                              Sim2 = as.data.frame(counts(dds_lun[1:10000, c(1:5, 84:89)])))
devtools::use_data(countsimExample_dfmat, pkg = "../..", overwrite = TRUE)

# Update data for Bioc 3.22 (not used in the end as it didn't make a difference)
data("countsimExample")
countsimExample <- list(
  Original = DESeqDataSetFromMatrix(
    countData = counts(countsimExample$Original),
    colData = colData(countsimExample$Original),
    design = ~ group),
  Sim1 = DESeqDataSetFromMatrix(
    countData = counts(countsimExample$Sim1),
    colData = colData(countsimExample$Sim1),
    design = ~ Group),
  Sim2 = DESeqDataSetFromMatrix(
    countData = counts(countsimExample$Sim2),
    colData = colData(countsimExample$Sim2),
    design = ~ Group)
)
usethis::use_data(countsimExample, overwrite = TRUE)

#' Calculate Spearman correlation between sample pairs
#'
#' @param ddsList List of lists, with one element per data set. Each element is
#'   a list containing a DGEList and a DESeqDataSet, with calculated
#'   dispersions (e.g., output from \code{\link{calculateDispersionsddsList}}).
#' @param maxNForCorr Maximal number of samples to use for correlation
#'   calculation. If the number of samples in a data set exceeds
#'   \code{maxNForCorr}, \code{maxNForCorr} samples will be randomly selected
#'   for calculation of correlations.
#'
#' @return A data frame with pairwise sample correlations for each data set
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
#' @importFrom stats cor
#' @importFrom edgeR cpm
#'
calculateSampleCorrs <- function(ddsList, maxNForCorr) {
  sampleCorrDF <- lapply(ddsList, function(x) {
    ## Calculate logCPMs
    cpms <- edgeR::cpm(x$dge, prior.count = 2, log = TRUE)

    ## Subsample columns if required
    if (ncol(cpms) > maxNForCorr) {
      cpms <- cpms[, sample(seq_len(ncol(cpms)), maxNForCorr, replace = FALSE)]
    }

    ## Calculate Spearman correlations
    corrs <- stats::cor(cpms, use = "pairwise.complete.obs",
                        method = "spearman")
    data.frame(
      Correlation = corrs[upper.tri(corrs)]
    )
  })
  ## Merge correlations from all data sets
  ns <- vapply(sampleCorrDF, nrow, 0)
  do.call(rbind, sampleCorrDF) %>%
    dplyr::mutate(dataset = rep(names(sampleCorrDF), ns))
}

#' Calculate Spearman correlation between feature pairs
#'
#' @param ddsList List of lists, with one element per data set. Each element is
#'   a list containing a DGEList and a DESeqDataSet, with calculated
#'   dispersions (e.g., output from \code{\link{calculateDispersionsddsList}}).
#' @param maxNForCorr Maximal number of features to use for calculation of
#'   correlations. If the number of features in a data set exceeds
#'   \code{maxNForCorr}, \code{maxNForCorr} features will be randomly selected
#'   for calculation of correlations.
#'
#' @return A data frame with pairwise feature correlations for each data set
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
#' @importFrom stats cor
#' @importFrom edgeR cpm
#' @importFrom genefilter rowVars
#'
calculateFeatureCorrs <- function(ddsList, maxNForCorr) {
  featureCorrDF <- lapply(ddsList, function(x) {
    ## Calculate logCPMs, keep only non-constant features
    cpms <- edgeR::cpm(x$dge, prior.count = 2, log = TRUE)
    cpms <- cpms[genefilter::rowVars(cpms) > 0, ]

    ## Subsample rows if required
    if (nrow(cpms) > maxNForCorr) {
      cpms <- cpms[sample(seq_len(nrow(cpms)), maxNForCorr, replace = FALSE), ]
    }

    ## Calculate Spearman correlations
    corrs <- stats::cor(t(cpms), use = "pairwise.complete.obs",
                        method = "spearman")
    data.frame(
      Correlation = corrs[upper.tri(corrs)]
    )
  })
  ## Merge correlations from all data sets
  ns <- vapply(featureCorrDF, nrow, 0)
  do.call(rbind, featureCorrDF) %>%
    dplyr::mutate(dataset = rep(names(featureCorrDF), ns))

}

library(countsimQC)
context("Check individual functions")

local({
  data(countsimExample)
  x <- lapply(countsimExample, function(w) w[1:50, ])
  set.seed(1)
  xsub <- calculateDispersionsddsList(x, maxNForDisp = 4)
  x <- calculateDispersionsddsList(x, maxNForDisp = Inf)

  test_that("Dimensions are correct", {
    expect_equivalent(unlist(lapply(x, function(w) c(nrow(w$dge), nrow(w$dds)))),
                      rep(50, 6))
    expect_equivalent(unlist(lapply(x, function(w) c(ncol(w$dge), ncol(w$dds)))),
                      rep(11, 6))
    expect_equivalent(unlist(lapply(xsub, function(w) c(nrow(w$dge), nrow(w$dds)))),
                      rep(50, 6))
    expect_equivalent(unlist(lapply(xsub, function(w) c(ncol(w$dge), ncol(w$dds)))),
                      rep(11, 6))
  })

  ## Sample correlations
  ## With calculateSampleCorrs
  set.seed(1)
  sample_corrs <- calculateSampleCorrs(x, maxNForCorr = 3)
  ## Manually
  set.seed(1)
  cpms <- edgeR::cpm(x$Original$dge, prior.count = 2, log = TRUE)
  samples_to_keep <- sample(seq_len(ncol(cpms)), 3, replace = FALSE)
  test_that("Sample correlations are correct", {
    expect_equal(subset(sample_corrs, dataset == "Original")$Correlation[1],
                 stats::cor(cpms[, samples_to_keep[1]], cpms[, samples_to_keep[2]],
                            method = "spearman", use = "pairwise.complete.obs"))
    expect_equal(subset(sample_corrs, dataset == "Original")$Correlation[2],
                 stats::cor(cpms[, samples_to_keep[1]], cpms[, samples_to_keep[3]],
                            method = "spearman", use = "pairwise.complete.obs"))
  })

  ## Feature correlations
  ## With calculateFeatureCorrs
  set.seed(1)
  feature_corrs <- calculateFeatureCorrs(x, maxNForCorr = 3)
  ## Manually
  set.seed(1)
  cpms <- edgeR::cpm(x$Original$dge, prior.count = 2, log = TRUE)
  cpms <- cpms[genefilter::rowVars(cpms) > 0, ]
  features_to_keep <- sample(seq_len(nrow(cpms)), 3, replace = FALSE)
  test_that("Feature correlations are correct", {
    expect_equal(subset(feature_corrs, dataset == "Original")$Correlation[1],
                 stats::cor(cpms[features_to_keep[1], ], cpms[features_to_keep[2], ],
                            method = "spearman", use = "pairwise.complete.obs"))
    expect_equal(subset(feature_corrs, dataset == "Original")$Correlation[2],
                 stats::cor(cpms[features_to_keep[1], ], cpms[features_to_keep[3], ],
                            method = "spearman", use = "pairwise.complete.obs"))
  })

  ## Stats
  sampleDF <- lapply(x, function(w) {
    data.frame(
      Libsize = colSums(w$dge$counts),
      Fraczero = colMeans(w$dge$counts == 0),
      TMM = w$dge$samples$norm.factors
    ) %>% dplyr::mutate(EffLibsize = Libsize * TMM)
  })
  ns <- sapply(sampleDF, nrow)
  sampleDF <- do.call(rbind, sampleDF) %>%
    dplyr::mutate(dataset = rep(names(sampleDF), ns))

  test_that("Summary stats are correct", {
    expect_equal(subset(sampleDF, dataset == "Sim1")$Fraczero[1],
                 mean(x$Sim1$dge$counts[, 1] == 0))
    expect_equal(subset(sampleDF, dataset == "Sim2")$Libsize[1],
                 sum(x$Sim2$dge$counts[, 1]))
    expect_equivalent(subset(sampleDF, dataset == "Sim2")$TMM[1],
                      edgeR::calcNormFactors(x$Sim2$dge$counts)[1])
    expect_equivalent(subset(sampleDF, dataset == "Sim2")$EffLibsize[1],
                      edgeR::calcNormFactors(x$Sim2$dge$counts)[1] *
                        sum(x$Sim2$dge$counts[, 1]))
  })

  expect_warning(
    statDF <- makeDF(df = sampleDF, column = "TMM",
                     permutationPvalues = FALSE, nPermutations = 0,
                     subsampleSize = 11, kmin = 3, kfrac = 0.1)$x$data)
  set.seed(1)
  expect_warning(
    statDFperm <- makeDF(df = sampleDF, column = "TMM",
                         permutationPvalues = TRUE, nPermutations = 100,
                         subsampleSize = 11, kmin = 3, kfrac = 0.1)$x$data)

  ## Average silhouette width is different since we're taking a subsample
  compare_cols <- setdiff(colnames(statDF), "Average silhouette width")
  test_that("Stats are the same whether permutation p-values are calculated or not", {
    expect_equal(statDF[, compare_cols], statDFperm[, compare_cols])
  })

  test_that("Derived stats are correct", {
    expect_equivalent(signif(stats::ks.test(sampleDF$TMM[sampleDF$dataset == "Original"],
                                            sampleDF$TMM[sampleDF$dataset == "Sim1"])$stat, 3),
                      statDF$`K-S statistic`[statDF$dataset1 == "Original" & statDF$dataset2 == "Sim1"])
    expect_equivalent(signif(stats::ks.test(sampleDF$TMM[sampleDF$dataset == "Original"],
                                            sampleDF$TMM[sampleDF$dataset == "Sim1"])$p.value, 3),
                      statDF$`K-S p-value`[statDF$dataset1 == "Original" & statDF$dataset2 == "Sim1"])
  })

  expect_equal(length(countsimQC:::defaultStats(n = 1, withP = FALSE)), 8L)
  expect_equal(length(countsimQC:::defaultStats(n = 1, withP = TRUE)), 12L)
  expect_equal(length(countsimQC:::defaultStats(n = 2, withP = FALSE)), 3L)
  expect_equal(length(countsimQC:::defaultStats(n = 2, withP = TRUE)), 6L)

  dtd <- countsimQC:::defineTableDesc(
    calculateStatistics = TRUE, subsampleSize = 5, kfrac = 0.25, kmin = 4,
    obstype = "sample", aspect = "aspect", minvalue = 0, maxvalue = 1,
    permutationPvalues = TRUE, nPermutations = 10, nDatasets = 3)
  expect_is(dtd, "list")
  expect_length(dtd, 2)
  expect_named(dtd, c("tabledesc", "tabledesc2d"), ignore.order = TRUE)

  dtdnostat <- countsimQC:::defineTableDesc(
    calculateStatistics = FALSE, subsampleSize = 5, kfrac = 0.25, kmin = 4,
    obstype = "sample", aspect = "aspect", minvalue = 0, maxvalue = 1,
    permutationPvalues = FALSE, nPermutations = 10, nDatasets = 3)
  expect_is(dtdnostat, "list")
  expect_length(dtdnostat, 2)
  expect_named(dtdnostat, c("tabledesc", "tabledesc2d"), ignore.order = TRUE)

  dtd <- countsimQC:::defineTableDesc(
    calculateStatistics = TRUE, subsampleSize = 5, kfrac = 0.25, kmin = 4,
    obstype = "sample", aspect = "aspect", minvalue = 0, maxvalue = 1,
    permutationPvalues = TRUE, nPermutations = 10, nDatasets = 1)
  expect_is(dtd, "list")
  expect_length(dtd, 2)
  expect_named(dtd, c("tabledesc", "tabledesc2d"), ignore.order = TRUE)
  expect_equal(dtd[[1]],
               "No statistics were calculated, since there is only one data set.")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE,
                      crop = NA)

## ----warning = FALSE----------------------------------------------------------
suppressPackageStartupMessages({
  library(countsimQC)
  library(DESeq2)
})

data(countsimExample)
countsimExample
countsimExample <- lapply(countsimExample, function(cse) {
  cse[seq_len(1500), ]
})

## ----results = "hide", warning = FALSE, message = FALSE-----------------------
tempDir <- tempdir()
countsimQCReport(ddsList = countsimExample, outputFile = "countsim_report.html",
                 outputDir = tempDir, outputFormat = "html_document", 
                 showCode = FALSE, forceOverwrite = TRUE,
                 savePlots = TRUE, description = "This is my test report.", 
                 maxNForCorr = 25, maxNForDisp = Inf, 
                 calculateStatistics = TRUE, subsampleSize = 25,
                 kfrac = 0.01, kmin = 5, 
                 permutationPvalues = FALSE, nPermutations = NULL)

## ----warning = FALSE----------------------------------------------------------
ggplots <- readRDS(file.path(tempDir, "countsim_report_ggplots.rds"))
if (!dir.exists(file.path(tempDir, "figures"))) {
  dir.create(file.path(tempDir, "figures"))
}
generateIndividualPlots(ggplots, device = "pdf", nDatasets = 3, 
                        outputDir = file.path(tempDir, "figures"))

## -----------------------------------------------------------------------------
data(countsimExample_dfmat)
names(countsimExample_dfmat)
lapply(countsimExample_dfmat, class)

## ----results = "hide", warning = FALSE, eval = FALSE--------------------------
# tempDir <- tempdir()
# countsimQCReport(ddsList = countsimExample_dfmat,
#                  outputFile = "countsim_report_dfmat.html",
#                  outputDir = tempDir, outputFormat = "html_document",
#                  showCode = FALSE, forceOverwrite = TRUE,
#                  savePlots = TRUE, description = "This is my test report.",
#                  maxNForCorr = 25, maxNForDisp = Inf,
#                  calculateStatistics = TRUE, subsampleSize = 25,
#                  kfrac = 0.01, kmin = 5,
#                  permutationPvalues = FALSE, nPermutations = NULL)

## -----------------------------------------------------------------------------
sessionInfo()

#' Construct data frame with pairwise statistics
#'
#' Construct a data frame containing statistics and p-values for pairwise
#' comparison of data sets.
#'
#' @param df The input data frame. Must contain at least a column named
#'   'dataset' and an additional column with values
#' @param column The name of the column(s) of \code{df} to be used as the basis
#'   for the comparison
#' @param permutationPvalues Whether or not to calculate p-values of statistics
#'   via permutation
#' @param nPermutations The number of permutations (only used if
#'   permutationPvalues = TRUE)
#' @param subsampleSize The number of observations for which certain
#'   (time-consuming) statistics will be calculated. The observations will be
#'   selected randomly among the rows of \code{df}
#' @param kmin,kfrac For statistics that require the extraction of k nearest
#'   neighbors of a given point, the number of neighbors will be max(kmin, kfrac
#'   * nrow(df))
#'
#' @return A data table with statistics and p-values for pairwise comparisons of
#'   data sets, based on the provided \code{column}
#'
#' @author Charlotte Soneson
#' @keywords internal
#'
#' @import dplyr
#' @importFrom utils combn
#'
makeDF <- function(df, column, permutationPvalues, nPermutations,
                   subsampleSize, kmin, kfrac) {
  ## Generate data frame with comparisons of all pairs of data sets
  if (length(unique(df$dataset)) > 1) {
    ## Initialize data frame by populating it with all data set pairs
    ds_df <- data.frame(t(utils::combn(unique(df$dataset), 2)),
                        stringsAsFactors = FALSE) %>%
      dplyr::rename(dataset1 = X1, dataset2 = X2)

    ## Calculate statistics for each pair of data sets
    ds_res <- as.data.frame(t(vapply(seq_len(nrow(ds_df)), function(i) {
      dftmp <- df %>% dplyr::filter(dataset %in%
                                      c(ds_df$dataset1[i], ds_df$dataset2[i]))

      obs_stats <- calculateStats(df = dftmp, ds1 = ds_df$dataset1[i],
                                  ds2 = ds_df$dataset2[i], column = column,
                                  subsampleSize = subsampleSize,
                                  permute = FALSE, kmin = kmin, kfrac = kfrac,
                                  xmax = max(df[, column], na.rm = TRUE),
                                  xmin = min(df[, column], na.rm = TRUE))
      if (permutationPvalues) {
        perm_stats <- t(vapply(seq_len(nPermutations), function(s) {
          calculateStats(df = dftmp, ds1 = ds_df$dataset1[i],
                         ds2 = ds_df$dataset2[i], column = column,
                         subsampleSize = subsampleSize,
                         permute = TRUE, kmin = kmin, kfrac = kfrac,
                         xmax = max(df[, column], na.rm = TRUE),
                         xmin = min(df[, column], na.rm = TRUE))
        }, defaultStats(length(column), withP = FALSE)))
        ## Permutation p-values
        obs_stats["NNmismatchP"] <-
          signif(mean(c(obs_stats["NNmismatch"], perm_stats[, "NNmismatch"]) >=
                        obs_stats["NNmismatch"]), 3)
        obs_stats["avesilhP"] <-
          signif(mean(c(obs_stats["avesilh"], perm_stats[, "avesilh"]) >=
                        obs_stats["avesilh"]), 3)
        obs_stats["avesilhlocalP"] <-
          signif(mean(c(obs_stats["avesilhlocal"],
                        perm_stats[, "avesilhlocal"]) >=
                        obs_stats["avesilhlocal"]), 3)
        if (length(column) == 1)
          obs_stats["diffareaP"] <-
          signif(mean(c(obs_stats["diffarea"], perm_stats[, "diffarea"]) >=
                        obs_stats["diffarea"]), 3)
      }
      obs_stats
    }, defaultStats(length(column), withP = permutationPvalues))))

    ## Populate the output table
    if (length(column) == 1) {
      ds_df$`K-S statistic` <- ds_res$ksstatistic
      ds_df$`K-S p-value` <- ds_res$kspvalue
      ds_df$`Scaled area between eCDFs` <- ds_res$diffarea
      if (permutationPvalues) {
        ds_df$`Scaled area between eCDFs perm p-value` <- ds_res$diffareaP
      }
      ds_df$`Runs statistic` <- ds_res$runsstatistic
      ds_df$`Runs p-value` <- ds_res$runspvalue
    }
    ds_df$`NN rejection fraction` <- ds_res$NNmismatch
    if (permutationPvalues) {
      ds_df$`NN rejection fraction perm p-value` <- ds_res$NNmismatchP
    }
    ds_df$`Average silhouette width` <- ds_res$avesilh
    if (permutationPvalues) {
      ds_df$`Average silhouette width perm p-value` <- ds_res$avesilhP
    }
    ds_df$`Average local silhouette width` <- ds_res$avesilhlocal
    if (permutationPvalues) {
      ds_df$`Average local silhouette width perm p-value` <-
        ds_res$avesilhlocalP
    }

    ## Return output table
    DT::datatable(ds_df, options = list(
      scrollX = TRUE,
      columnDefs = list(list(className = "dt-center", targets = 3:ncol(ds_df)))
    ))
  }
}

#' Define table descriptions
#'
#' Generate the text that describes the content of the tables generated by
#' \code{makeDF}.
#'
#' @param calculateStatistics Whether or not statistics and p-values are
#'   calculated
#' @param subsampleSize The number of observations for which certain
#'   (time-consuming) statistics will be calculated
#' @param kmin,kfrac For statistics that require the extraction of k nearest
#'   neighbors of a given point, the number of neighbors will be max(kmin, kfrac
#'   * nrow(df))
#' @param obstype The type of observation (e.g., sample, feature, sample pair)
#' @param aspect The name of the aspect of interest
#' @param minvalue,maxvalue The minimal and maximal value of the aspect of
#'   interest, used for scaling of the x axis when calculating the area between
#'   the eCDFs
#' @param permutationPvalues Whether or not to calculate p-values of statistics
#'   via permutation
#' @param nPermutations The number of permutations (only used if
#'   permutationPvalues = TRUE)
#' @param nDatasets The number of data sets that are being compared
#'
#' @return A list with two text strings in markdown format: one for tables based
#'   on a single data column, and one for tables based on two data columns
#'
#' @keywords internal
#' @author Charlotte Soneson
#'
defineTableDesc <- function(calculateStatistics, subsampleSize, kfrac, kmin,
                            obstype, aspect, minvalue, maxvalue,
                            permutationPvalues, nPermutations, nDatasets) {
  if (nDatasets < 2) {
    tabledesc <- tabledesc2d <-
      "No statistics were calculated, since there is only one data set."
  } else {
    if (calculateStatistics) {
      if (permutationPvalues) {
        permtext <- paste0("The permutation p-values below were estimated ",
                           "based on ", nPermutations, " permutations.")
      } else {
        permtext <- ""
      }
      tabledesc <- paste0("The table below contains a range of quantitative ",
                          "statistics and test results evaluating the degree ",
                          "of similarity between each pair of data sets based ",
                          "on the ", aspect, ". The following statistics and ",
                          "test results are included:

- *K-S statistic/K-S p-value*: the [Kolmogorov-Smirnov statistic]",
"(https://en.wikipedia.org/wiki/Kolmogorov%E2%80%93Smirnov_test) ",
"[@Kolmogorov1933, @Smirnov1948], i.e., the maximal distance between the two ",
"empirical cumulative distribution functions (eCDFs), and the associated ",
"p-value.
- *Scaled area between eCDFs*: the absolute area between the eCDFs for the ",
"two data sets. This is calculated by first subtracting one eCDF from the ",
"other and taking the absolute value of the differences, followed by scaling ",
"the x-axis so that the difference between the largest and smallest value ",
"across all data sets is equal to 1 (here, subtracting ", minvalue, " and ",
"dividing by ", maxvalue - minvalue, "), and finally calculating the area ",
"under the resulting non-negative curve. This gives a scaled area with ",
"values in the range [0, 1], where large values indicate large differences ",
"between the ", aspect, " distributions.
- *Runs statistic/Runs p-value*: the statistic and p-value from a one-sided ",
"[Wald-Wolfowitz runs test]",
"(https://en.wikipedia.org/wiki/Wald%E2%80%93Wolfowitz_runs_test) ",
"[@WaldWolfowitz1940]. The values from both data sets are first sorted in ",
"increasing order and converted into a binary sequence based on whether or ",
"not they are derived from the first data set. The runs test is applied to ",
"this sequence. The test is one-sided, with the alternative hypothesis being ",
"that there is a trend towards values from the same data set occurring next ",
"to each other in the sequence.
- *NN rejection fraction*: this statistic is similar to one used by the ",
"[kBET](https://github.com/theislab/kBET) R package. First, a subset of R ",
obstype, "s are selected (where R is chosen to be the smallest of ",
subsampleSize, " and the total number of ", obstype, "s in the two data sets)",
", and for each of these ", obstype, "s the k nearest neighbors are found ",
"(where k is chosen to be ", kfrac * 100, "% of the total number of ", obstype,
"s from the two data sets, however at least ", kmin, "). Then, a chi-square ",
"test is used to evaluate whether the composition of data sets from which ",
"the k nearest neighbors of each selected ", obstype, "s comes differs ",
"significantly from the overall composition of ", obstype, "s from the two ",
"data sets. The NN rejection fraction is the fraction of selected ", obstype,
"s for which the chi-square test rejects the null hypothesis at a ",
"significance level of 5%.
- *Average silhouette width*: as for the NN rejection fraction, a subset of R ",
obstype, "s are selected. For each of these, the (Euclidean) distances to all ",
"other ", obstype, "s are calculated, and the [silhouette width]",
"(https://en.wikipedia.org/wiki/Silhouette_(clustering)) ",
"[@Rousseeuw1987silhouette] is calculated based on these distances. The final",
" statistic is the average silhouette width for the R ", obstype, "s.
- *Average local silhouette width*: similar to the average silhouette width, ",
"but for each ", obstype, ", only the k' closest ", obstype, "s from a data ",
"set are used to calculate the silhouette widths. Here, k' for data set i is ",
"defined as k * p_i, where k is as above and p_i is the fraction of the total ",
"number of ", obstype, "s that come from data set i.
- For the scaled area between eCDFs, NN rejection fraction and average ",
"silhouette widths, permutation p-values can be calculated by permuting the ",
"data set labels. These values are only available if the 'permutationPvalues' ",
"argument to 'countsimQCReport' was set to TRUE. ", permtext,"

It should be noted that the reported statistics in some cases are highly ",
"dependent on the number of underlying observations, and especially with a ",
"large number of observations, very small p-values can correspond to small ",
"effective differences. Thus, interpretation should ideally be guided by both ",
"quantitative and qualitative, visual information.")
      tabledesc2d <- paste0("The table below contains a range of quantitative ",
"statistics and test results evaluating the degree of similarity between each ",
"pair of data sets based on the ", aspect, ". The following statistics and ",
"test results are included:

- *NN rejection fraction*: this statistic is similar to one used by the ",
"[kBET](https://github.com/theislab/kBET) R package. First, a subset of R ",
obstype, "s are selected (where R is chosen to be the smallest of ",
subsampleSize, " and the total number of ", obstype, "s in the two data sets)",
", and for each of these ", obstype, "s the k nearest neighbors are found ",
"(where k is chosen to be ", kfrac * 100, "% of the total number of ", obstype,
"s from the two data sets, however at least ", kmin, "). Then, a chi-square ",
"test is used to evaluate whether the composition of data sets from which the ",
"k nearest neighbors of each selected ", obstype, "s comes differs ",
"significantly from the overall composition of ", obstype, "s from the two ",
"data sets. The NN rejection fraction is the fraction of selected ", obstype,
"s for which the chi-square test rejects the null hypothesis at a ",
"significance level of 5%.
- *Average silhouette width*: as for the NN rejection fraction, a subset of R ",
obstype, "s are selected. For each of these, the (Euclidean) distances to ",
"all other ", obstype, "s are calculated, and the [silhouette width]",
"(https://en.wikipedia.org/wiki/Silhouette_(clustering)) ",
"[@Rousseeuw1987silhouette] is calculated based on these distances. The final ",
"statistic is the average silhouette width for the R ", obstype, "s.
- *Average local silhouette width*: similar to the average silhouette width, ",
"but for each ", obstype, ", only the k' closest ", obstype, "s from a data ",
"set are used to calculate the silhouette widths. Here, k' for data set i is ",
"defined as k * p_i, where k is as above and p_i is the fraction of the ",
"total number of ", obstype, "s that come from data set i.
- For the NN rejection fraction and average silhouette widths, permutation ",
"p-values can be calculated by permuting the data set labels. These values ",
"are only available if the 'permutationPvalues' argument to ",
"'countsimQCReport' was set to TRUE. ", permtext)
    } else {
      tabledesc <- tabledesc2d <-
        paste0("No statistics were calculated, since the ",
               "'calculateStatistics' argument to 'countsimQCReport()' was ",
               "set to FALSE. To perform pairwise quantitative comparisons ",
               "between data sets, set this argument to TRUE. Note, however, ",
               "that this increases the runtime significantly.")
    }
  }

  list(tabledesc = tabledesc, tabledesc2d = tabledesc2d)
}

#' Check whether pandoc and pandoc-citeproc is available
#'
#' @author Charlotte Soneson
#'
#' @param ignorePandoc logical. If TRUE, just give a warning if pandoc or
#'   pandoc-citeproc is not available. If FALSE, stop.
#'
#' @keywords internal
#'
#' @return A logical(1), indicating whether pandoc can be run or not.
#'   In addition, raises either a warning or an error (depending on the
#'   value of \code{ignorePandoc}) if pandoc or pandoc-citeproc is not
#'   available.
#'
#' @importFrom rmarkdown pandoc_available pandoc_exec
#'
.checkPandoc <- function(ignorePandoc) {
  ## Initialize output to TRUE
  doRender <- TRUE

  ## First check whether pandoc is available
  if (!rmarkdown::pandoc_available()) {
    doRender <- FALSE
    ## If pandoc is not available, either give a warning or an error,
    ## depending on the value of ignorePandoc
    if (ignorePandoc) {
      ## If ignorePandoc is TRUE, just give a warning
      warning("pandoc is not available! ",
              "The final report will not be generated.",
              immediate. = TRUE)
    } else {
      ## If ignorePandoc is FALSE, stop
      stop("pandoc is not available!")
    }
  } else {
    ## If pandoc is available, check for pandoc-citeproc
    ## Only do this if the pandoc version is <2.11, since
    ## pandoc-citeproc is not included (or needed) in v2.11 and later.
    if (!rmarkdown::pandoc_available(version = "2.11")) {
      ## TRUE if the available pandoc version is not 2.11 or newer
      ## pandoc-citeproc should be found in the path, or in the
      ## same folder as the pandoc executable
      if (Sys.which("pandoc-citeproc") == "" &&
          !file.exists(file.path(dirname(rmarkdown::pandoc_exec()),
                                 "pandoc-citeproc"))) {
        doRender <- FALSE
        ## pandoc-citeproc is required, but not found
        if (ignorePandoc) {
          ## If ignorePandoc is TRUE, just give a warning
          warning("pandoc-citeproc is not available! ",
                  "The final report will not be generated.",
                  immediate. = TRUE)
        } else {
          ## If ignorePandoc is FALSE, stop
          stop("pandoc-citeproc is not available!")
        }
      }
    }
  }
  return(doRender)
}

#' Generate countsimQC report
#'
#' Generate a report comparing a range of characteristics across a collection of
#' one or more count data sets.
#'
#' @param ddsList Named list of \code{DESeqDataSets} or count matrices to
#'   compare. See the \code{DESeq2} Bioconductor package
#'   (http://bioconductor.org/packages/release/bioc/html/DESeq2.html) for more
#'   information about the \code{DESeqDataSet} class. Each \code{DESeqDataSet}
#'   object in the list should contain a count matrix, a data frame with sample
#'   information and a design formula. The sample information and design formula
#'   will be used to calculate dispersions appropriately. If count matrices are
#'   provided, it is assumed that all columns represent replicate samples, and
#'   the design formula ~1 will be used.
#' @param outputFile The file name of the final report. The extension must match
#'   the selected \code{outputFormat} (i.e., either .html or .pdf).
#' @param outputDir The directory where the final report should be saved.
#' @param outputFormat The output format of the report. If set to NULL or
#'   "html_document", an html report will be generated. If set to
#'   "pdf_document", a pdf report will be generated.
#' @param showCode Whether or not to include the code in the final report.
#' @param rmdTemplate The Rmarkdown (.Rmd) file that will be used as the
#'   template for generating the report. If set to NULL (default), the template
#'   provided with the countsimQC package will be used. See Details for more
#'   information.
#' @param forceOverwrite Whether to force overwrite existing output files when
#'   saving the generated report and figures.
#' @param savePlots Whether to save the ggplot objects for all the output
#'   figures, to allow additional fine-tuning and generation of individual
#'   plots. Note that the resulting file can be quite large, especially when
#'   many and/or large data sets are compared.
#' @param description A string (of arbitrary length) describing the content of
#'   the generated report. This will be included in the beginning of the report.
#'   If set to NULL, a default description listing the number and names of the
#'   included data sets will be used.
#' @param maxNForCorr The maximal number of samples (features) for which
#'   pairwise correlation coefficients will be calculated. If the number of
#'   samples (features) exceeds this number, they will be randomly subsampled.
#' @param maxNForDisp The maximal number of samples that will be used to
#'   estimate dispersions. By default, all samples are used. This can be lowered
#'   to speed up calculations (and obtain approximate results) for large data
#'   sets.
#' @param calculateStatistics Whether to calculate quantitative pairwise
#'   statistics for comparing data sets in addition to generating the plots.
#' @param subsampleSize The number of randomly selected observations (samples,
#'   features or pairs of samples or features) for which certain
#'   (time-consuming) statistics will be calculated. Only used if
#'   \code{calculateStatistics} = TRUE.
#' @param kmin,kfrac For statistics that require the extraction of the k nearest
#'   neighbors of a given point, the number of neighbors will be max(kmin, kfrac
#'   * nrow(df))
#' @param permutationPvalues Whether to calculate permutation p-values for
#'   selected pairwise data set comparison statistics.
#' @param nPermutations The number of permutations to perform when calculating
#'   permutation p-values for data set comparison statistics. Only used if
#'   \code{permutationPvalues} = TRUE.
#' @param knitrProgress Whether to show the progress bar when the report is
#'   generated.
#' @param quiet Whether to suppress warnings and progress messages when the
#'   report is generated.
#' @param ignorePandoc Determines what to do if pandoc or pandoc-citeproc is
#'   missing (if Sys.which("pandoc") or Sys.which("pandoc-citeproc") is ""). If
#'   ignorePandoc is TRUE, only a warning is given. The figures will be
#'   generated, but not the final report. If ignorePandoc is FALSE (default),
#'   the execution stops immediately.
#' @param useRAGG Logical scalar, indicating whether to use ragg_png as the
#'   graphics device in the report rather than the default png.
#' @param dpi Numeric scalar, setting the dpi of the generated plots. Only
#'   used if \code{useRAGG} is \code{TRUE}.
#' @param ... Other arguments that will be passed to \code{rmarkdown::render}.
#'
#' @author Charlotte Soneson
#'
#' @details When the function is called, the template file (specified by
#'   \code{rmdTemplate}) will be copied into the output folder, and
#'   \code{rmarkdown::render} will be called to generate the final report. If
#'   there is already a .Rmd file with the same name in the output folder, the
#'   function will raise an error and stop, to avoid overwriting the existing
#'   file. The reason for this behaviour is that the copied template in the
#'   output folder will be deleted once the report is generated.
#'
#' @export
#' @importFrom rmarkdown render
#' @importFrom tools file_ext file_path_sans_ext
#' @importFrom caTools trapz
#' @importFrom randtests runs.test
#' @importFrom methods is
#' @import SummarizedExperiment DESeq2 edgeR dplyr tidyr ggplot2
#' @import genefilter DT GenomeInfoDbData ragg
#' @return No value is returned, but a report is generated in the
#'   \code{outputDir} directory.
#'
#' @examples
#' ## Load example data
#' data(countsimExample)
#' \dontrun{
#' ## Generate report
#' countsimQCReport(countsimExample, outputDir = "./",
#'                  outputFile = "example.html")
#' }
#'
countsimQCReport <- function(ddsList, outputFile, outputDir = "./",
                             outputFormat = NULL, showCode = FALSE,
                             rmdTemplate = NULL, forceOverwrite = FALSE,
                             savePlots = FALSE, description = NULL,
                             maxNForCorr = 500, maxNForDisp = Inf,
                             calculateStatistics = TRUE, subsampleSize = 500,
                             kfrac = 0.01, kmin = 5,
                             permutationPvalues = FALSE, nPermutations = NULL,
                             knitrProgress = FALSE, quiet = FALSE,
                             ignorePandoc = FALSE, useRAGG = FALSE,
                             dpi = 96, ...){
  ## This function was inspired by code from Nicholas Hamilton, provided at
  ## http://stackoverflow.com/questions/37097535/generate-report-in-r

  if (is.null(outputFormat)) outputFormat <- "html_document"

  doRender <- .checkPandoc(ignorePandoc)

  ## ------------------------------------------------------------------------ ##
  ## --------------------- Check input arguments ---------------------------- ##
  ## ------------------------------------------------------------------------ ##

  ## ------------------------ outputFormat --------------------------------- ##
  ## Raise an error if outputFormat is not one of the allowed
  if (!(outputFormat %in% c("pdf_document", "html_document"))) {
    stop("The provided outputFormat is currently not supported. Please use ",
         "either 'html_document' (or NULL) or 'pdf_document'.", call. = FALSE)
  }

  ## Raise an error if the output format and file name extension don't match
  if (outputFormat != paste0(tools::file_ext(outputFile), "_document")) {
    stop(paste0("File name extension of outputFile doesn't agree with the ",
                "outputFormat, should be .",
                gsub("_document$", "", outputFormat)), call. = FALSE)
  }

  ## --------------------------- ddsList ----------------------------------- ##
  ## Raise an error if ddsList is not a named list of DESeqDataSets/data
  ## frames/matrices
  if (!is(ddsList, "list")) {
    stop("ddsList must be a list.", call. = FALSE)
  }
  if (length(setdiff(unique(names(ddsList)),
                     c("", NA, NULL))) != length(ddsList)) {
    stop("ddsList must be a named list, ",
         "with a unique name for each element.", call. = FALSE)
  }
  if (!all(vapply(ddsList, function(w) {
    is(w, "DESeqDataSet") | is(w, "data.frame") | is(w, "matrix")
  }, FALSE))) {
    stop("All elements of ddsList must be DESeqDataSet objects, data.frames ",
         "or matrices. See the DESeq2 Bioconductor package ",
         "(http://bioconductor.org/packages/release/bioc/html/DESeq2.html) ",
         "for more information about the DESeqDataSet class.", call. = FALSE)
  }

  ## If some objects are data frames or matrices, convert them into
  ## DESeqDataSets
  ddsList <- lapply(ddsList, function(ds) {
    if (is(ds, "DESeqDataSet")) {
      ds
    } else {
      DESeq2::DESeqDataSetFromMatrix(
        countData = round(as.matrix(ds)),
        colData = data.frame(sample = seq_len(ncol(ds))),
        design = ~ 1)
    }
  })
  stopifnot(all(vapply(ddsList, function(w) is(w, "DESeqDataSet"), FALSE)))

  ## ------------------------- output files --------------------------------- ##
  outputReport <- file.path(outputDir, basename(outputFile))
  outputPlots <- file.path(
    outputDir,
    paste0(tools::file_path_sans_ext(basename(outputFile)), "_ggplots.rds"))
  outputRmd <- file.path(
    outputDir,
    paste0(tools::file_path_sans_ext(basename(outputFile)), ".Rmd"))

  ## Report
  if (file.exists(outputReport)) {
    if (!forceOverwrite) {
      stop("The file ", outputReport,
           " already exists. Please remove or rename the file, provide ",
           "another value of outputFile, or set forceOverwrite = TRUE.",
           call. = FALSE)
    } else {
      if (!quiet) {
        warning("The file ", outputReport,
                " already exists and will be overwritten, since ",
                "forceOverwrite = TRUE.", immediate. = TRUE, call. = FALSE)
      }
    }
  }

  ## ggplot2 objects
  if (file.exists(outputPlots)) {
    if (savePlots & !forceOverwrite) {
      stop("The file ", outputPlots,
           ", where the ggplot objects will be generated, already exists. ",
           "Please remove or rename the file, provide another value ",
           "of outputFile, or set forceOverwrite = TRUE.", call. = FALSE)
    } else if (savePlots & forceOverwrite) {
      if (!quiet) {
        warning("The file ", outputPlots,
                ", where the ggplot objects will be generated, already exists ",
                "and will be overwritten, since forceOverwrite = TRUE.",
                immediate. = TRUE, call. = FALSE)
      }
    } else if (!savePlots) {
      if (!quiet) {
        warning("The file ", outputPlots,
                " already exists, but will not be overwritten ",
                "since savePlots = FALSE. Note that the ggplots in the ",
                "existing file may not correspond to the newly ",
                "generated report.", immediate. = TRUE, call. = FALSE)
      }
    }
  }

  ## ------------------------- Rmd template --------------------------------- ##
  ## Path to the template file
  if (is.null(rmdTemplate)) {
    templateFile <- system.file("extdata",
                                 "simulation_comparison_template_list.Rmd",
                                 package = "countsimQC")
  } else {
    templateFile <- rmdTemplate
  }
  if (file.exists(templateFile)) {
    if (file.exists(outputRmd)) {
      stop("There is already an .Rmd file ", outputRmd,
           ". Please remove or rename this file, or choose another ",
           "outputFile name.", call. = FALSE)
    } else {
      file.copy(from = templateFile, to = outputRmd, overwrite = FALSE)
    }
  } else {
    stop("The intended Rmd template file ", templateFile, " does not exist.",
         call. = FALSE)
  }

  ## ------------------------------------------------------------------------ ##
  ## ----------------- Add description if not provided ---------------------- ##
  ## ------------------------------------------------------------------------ ##

  if (is.null(description)) {
    description <- sprintf(
      "This report shows the characteristics of %s %s, named %s.",
      length(ddsList),
      ifelse(length(ddsList) == 1, "data set", "data sets"),
      paste(names(ddsList), collapse = ", "))
  }

  ## ------------------------------------------------------------------------ ##
  ## ----------------------- Process the arguments -------------------------- ##
  ## ------------------------------------------------------------------------ ##

  args <- list(...)
  args$input <- outputRmd
  args$output_format <- outputFormat
  args$output_file <- outputFile
  args$quiet <- !knitrProgress
  args$run_pandoc <- doRender

  ## ------------------------------------------------------------------------ ##
  ## ------------------------ Render the report ----------------------------- ##
  ## ------------------------------------------------------------------------ ##

  outputFile <- do.call("render", args = args)

  ## ------------------------------------------------------------------------ ##
  ## --------------------- Remove temporary file ---------------------------- ##
  ## ------------------------------------------------------------------------ ##

  file.remove(outputRmd)

  invisible(outputFile)
}

library(countsimQC)
context("Check report generation")

local({
  tempDir <- tempdir()
  data(countsimExample)
  x <- lapply(countsimExample, function(w) w[1:25, ])
  if (file.exists(file.path(tempDir, "report_from_test.Rmd"))) {
    file.remove(file.path(tempDir, "report_from_test.Rmd"))
  }
  if (file.exists(file.path(tempDir, "report_from_test.html"))) {
    file.remove(file.path(tempDir, "report_from_test.html"))
  }
  if (file.exists(file.path(tempDir, "report_from_test.pdf"))) {
    file.remove(file.path(tempDir, "report_from_test.pdf"))
  }
  if (file.exists(file.path(tempDir, "report_from_test_ggplots.rds"))) {
    file.remove(file.path(tempDir, "report_from_test_ggplots.rds"))
  }

  test_that("Input argument errors are recognized", {
    ## outputFormat
    expect_error(countsimQCReport(ddsList = x, outputFormat = "html",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE))
    expect_error(countsimQCReport(ddsList = x, outputFormat = "html_document",
                                  outputFile = "report_from_test.pdf",
                                  outputDir = tempDir, savePlots = TRUE))
    ## ddsList
    expect_error(countsimQCReport(ddsList = x[[1]], outputFormat = "html_document",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE))
    expect_error(countsimQCReport(ddsList = list(A = 1:3),
                                  outputFormat = "html_document",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE))
    y <- x
    names(y) <- NULL
    expect_error(countsimQCReport(ddsList = y, outputFormat = "html_document",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE))
    y <- x
    names(y) <- c("A", "A", "B")
    expect_error(countsimQCReport(ddsList = y, outputFormat = "html_document",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE))

    ## rmd_template
    expect_error(countsimQCReport(ddsList = x, outputFormat = "html_document",
                                  outputFile = "report_from_test.html",
                                  outputDir = tempDir, savePlots = TRUE,
                                  rmdTemplate = "nonexistent.Rmd"))

    ## existing rmd output
    file.create(file.path(tempDir, "test_generated.Rmd"))
    expect_error(countsimQCReport(ddsList = x, outputFormat = "html_document",
                                  outputFile = "test_generated.html",
                                  outputDir = tempDir, savePlots = TRUE))
    file.remove(file.path(tempDir, "test_generated.Rmd"))

    ## existing plot output
    file.create(file.path(tempDir, "test_generated2_ggplots.rds"))
    expect_error(countsimQCReport(ddsList = x, outputFormat = "html_document",
                                  outputFile = "test_generated2.html",
                                  outputDir = tempDir, savePlots = TRUE,
                                  forceOverwrite = FALSE))
    file.remove(file.path(tempDir, "test_generated2_ggplots.rds"))
  })

  ## This doesn't generate a warning ("Chi-squared approximation may be incorrect")
  ## anymore (starting from 2022-12-15).
  ## Also the progress messages are not displayed in the console anymore.
  ## Maybe due to changes in rmarkdown 2.19? They are still displayed with 2.18.
  wns <- capture_warnings({
    cqr <- countsimQCReport(ddsList = x, outputFormat = "html_document",
                            outputFile = "test_generated.html",
                            outputDir = tempDir, savePlots = TRUE,
                            description = NULL)
    })
  expect_is(cqr, "character")
  expect_error(countsimQCReport(ddsList = x, outputFormat = NULL,
                                outputFile = "test_generated.html",
                                outputDir = tempDir, savePlots = TRUE,
                                forceOverwrite = FALSE))
  ## Test that existing report will be overwritten if requested
  # expect_warning(
  #   cqr <- countsimQCReport(ddsList = x, outputFormat = NULL,
  #                           outputFile = "test_generated.html",
  #                           outputDir = tempDir, savePlots = TRUE,
  #                           forceOverwrite = TRUE))
  # expect_is(cqr, "character")

  # expect_warning(generateIndividualPlots(
  #   ggplotsRds = file.path(tempDir, "test_generated_ggplots.rds"),
  #   device = "png",
  #   outputDir = tempDir, nDatasets = 3
  # ))
  expect_warning(generateIndividualPlots(
    ggplotsRds = file.path(tempDir, "test_generated_ggplots.rds"),
    device = "png",
    outputDir = file.path(tempDir, "subdir"), nDatasets = 3
  ))

  expect_error(generateIndividualPlots(
    ggplotsRds = 1, device = "png",
    outputDir = file.path(tempDir, "subdir"), nDatasets = 3
  ), "The provided ggplotsRds object")
  expect_error(generateIndividualPlots(
    ggplotsRds = list(1, 2), device = "png",
    outputDir = file.path(tempDir, "subdir"), nDatasets = 3
  ), "The elements of the provided ggplotsRds object")

  expect_warning(countsimQCReport(ddsList = x, outputFormat = NULL,
                                  outputFile = "test_generated.html",
                                  outputDir = tempDir, savePlots = FALSE,
                                  forceOverwrite = TRUE),
                 "already exists, but will not be overwritten")

  ## Run with matrix input
  expect_warning(
    cqr <- countsimQCReport(ddsList = lapply(x, function(y) DESeq2::counts(y)),
                            outputFormat = NULL,
                            outputFile = "test_generated.html",
                            outputDir = tempDir, savePlots = FALSE,
                            forceOverwrite = TRUE))
  expect_is(cqr, "character")

  if (file.exists(file.path(tempDir, "report_from_test.html"))) {
    file.remove(file.path(tempDir, "report_from_test.html"))
  }
  if (file.exists(file.path(tempDir, "report_from_test.pdf"))) {
    file.remove(file.path(tempDir, "report_from_test.pdf"))
  }
  if (file.exists(file.path(tempDir, "report_from_test_ggplots.rds"))) {
    file.remove(file.path(tempDir, "report_from_test_ggplots.rds"))
  }
})

#' Example list with three count data sets in different formats
#'
#' A named list with three elements, each corresponding to a (real or simulated)
#' count data set. One of them is provided as a DESeqDataset, one as a count
#' data frame and one as a count matrix.
#'
#' The \code{Original} data set represents a subset of 10,000 genes and 11 cells
#' from the GSE74596 single-cell RNA-seq data set, obtained from the conquer
#' repository (http://imlspenticton.uzh.ch:3838/conquer/). The \code{Sim1} and
#' \code{Sim2} data sets similarly represent subsets of scRNA-seq data sets
#' simulated with two different simulation methods, using the real GSE74596 data
#' set as the basis for parameter estimation.

#' @format A named list with three elements, each corresponding to a (real or
#'   simulated) count data set.
#'
#' @return A named list with three elements, each corresponding to a (real or
#'   simulated) count data set.
#'
#' @usage data("countsimExample_dfmat")
"countsimExample_dfmat"

#' Generate individual plots from countsimQCReport output
#'
#' Generate separate plots for all evaluation criteria using the collection of
#' ggplot objects that can be saved when generating a countsimQC report (by
#' setting \code{savePlots = TRUE}).
#'
#' @param ggplotsRds The path to a .rds file generated by
#'   \code{countsimQCReport} by setting \code{savePlots = TRUE}, or the list of
#'   plots stored in this file.
#' @param device One of "eps", "ps", "tex" (pictex), "pdf", "jpeg", "tiff",
#'   "png", "bmp", "svg" or "wmf" (windows only) (will be provided to the
#'   \code{ggsave} function from the \code{ggplot2} package).
#' @param outputDir The output directory where the plots should be generated.
#' @param nDatasets The number of data sets that are compared in the figures.
#'   This is needed to set the size of the plots correctly.
#'
#' @author Charlotte Soneson
#'
#' @export
#'
#' @importFrom grDevices n2mfrow
#' @importFrom methods is
#' @importFrom ggplot2 is_ggplot
#'
#' @return Nothing is returned, but plots are generated in the designated
#'   output directory.
#'
#' @examples
#' ## Load example data
#' data(countsimExample)
#' \dontrun{
#' ## Generate report
#' countsimQCReport(countsimExample, outputDir = "./",
#'                  outputFile = "example.html", savePlots = TRUE)
#' ## Generate individual plots
#' generateIndividualPlots("example_ggplots.rds", nDatasets = 3)
#' }
#'
generateIndividualPlots <- function(ggplotsRds, device = "png",
                                    outputDir = "./", nDatasets = 2) {
  ## ------------------------------------------------------------------------ ##
  ## --------------------- Check input arguments ---------------------------- ##
  ## ------------------------------------------------------------------------ ##

  if (is(ggplotsRds, "character")) {
    ggplotsRds <- readRDS(ggplotsRds)
  }
  if (!is(ggplotsRds, "list")) {
    stop("The provided ggplotsRds object, or the object stored ",
         "in the ggplotsRds file, must be a list.")
  }
  if (!all(vapply(ggplotsRds, function(w) is_ggplot(w), FALSE))) {
    stop("The elements of the provided ggplotsRds object, or the object ",
         "stored in the ggplotsRds file, must be ggplot objects. ")
  }

  ## ------------------------------------------------------------------------ ##
  ## --------------------- Create output directory -------------------------- ##
  ## ------------------------------------------------------------------------ ##

  if (!dir.exists(outputDir)) {
    dir.create(outputDir, recursive = TRUE)
  }

  ## ------------------------------------------------------------------------ ##
  ## ------------------------- Generate plots ------------------------------- ##
  ## ------------------------------------------------------------------------ ##

  colRow <- grDevices::n2mfrow(nDatasets)
  for (pl in names(ggplotsRds)) {
    if (any(vapply(c("BCVedgeR", "dispersionDESeq2", "SepHist", "SepScatter"),
                   function(x) length(grep(x, pl)) > 0, FALSE))) {
      ggsave(plot = ggplotsRds[[pl]], filename = paste0(pl, ".", device),
             path = outputDir, width = 4 * colRow[1],
             height = 4 * colRow[2], units = "in")
    } else {
      ggsave(plot = ggplotsRds[[pl]], filename = paste0(pl, ".", device),
             path = outputDir, width = 7, height = 5, units = "in")
    }
  }
}

---
title: "Comparison of characteristic features across count data sets"
author: ""
date: ""
output: 
  html_document:
    toc: true
    toc_float: true
    code_folding: show
  pdf_document:
    toc: true
always_allow_html: yes
references:
- id: Love2014DESeq2
  title: Moderated estimation of fold change and dispersion for RNA-seq data with DESeq2
  author:
  - family: Love
    given: Michael I
  - family: Huber
    given: Wolfgang
  - family: Anders
    given: Simon
  container-title: Genome Biology
  volume: 15
  page: 550
  type: article-journal
  URL: https://genomebiology.biomedcentral.com/articles/10.1186/s13059-014-0550-8
  issued:
    year: 2014
- id: Robinson2010edgeR
  title: edgeR-a Bioconductor package for differential expression analysis of digital gene expression data
  author:
  - family: Robinson
    given: Mark D
  - family: McCarthy
    given: Davis J
  - family: Smyth
    given: Gordon K
  container-title: Bioinformatics
  volume: 26
  page: 139-140
  type: article-journal
  URL: https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btp616
  issued:
    year: 2010
- id: Robinson2010TMM
  title: A scaling normalization method for differential expression analysis of RNA-seq data
  author:
  - family: Robinson
    given: Mark D
  - family: Oshlack
    given: Alicia
  container-title: Genome Biology
  volume: 11
  page: R25
  type: article-journal
  URL: https://genomebiology.biomedcentral.com/articles/10.1186/gb-2010-11-3-r25
  issued:
    year: 2010
- id: WaldWolfowitz1940
  title: On a test whether two samples are from the same population
  author:
  - family: Wald
    given: Abraham
  - family: Wolfowitz
    given: Jacob
  container-title: The Annals of Mathematical Statistics
  volume: 11
  page: 147-162
  type: article-journal
  URL: https://projecteuclid.org/euclid.aoms/1177731909
  issued:
    year: 1940
- id: Kolmogorov1933
  title: Sulla determinazione empirica di una legge di distribuzione
  author:
  - family: Kolmogorov
    given: Andrey
  container-title: Giornale dell'Istituto Italiano degli Attuari
  volume: 4
  page: 83-91
  type: article-journal
  issued:
    year: 1933
- id: Smirnov1948
  title: Table for estimating the goodness of fit of empirical distributions
  author:
  - family: Smirnov
    given: Nikolai Vasilyevich
  container-title: Annals of Mathematical Statistics
  volume: 19
  page: 279-281
  type: article-journal
  URL: https://projecteuclid.org/euclid.aoms/1177730256
  issued:
    year: 1948
- id: Rousseeuw1987silhouette
  title: Silhouettes a graphical aid to the interpretation and validation of cluster analysis
  author:
  - family: Rousseeuw
    given: Peter J
  container-title: Journal of Computational and Applied Mathematics
  volumn: 20
  page: 53-65
  type: article-journal
  URL: http://www.sciencedirect.com/science/article/pii/0377042787901257
  issued:
    year: 1987
- id: Chen2014Dispersion
  title: Differential expression analysis of complex RNA-seq experiments using edgeR
  author:
  - family: Chen
    given: Yunshun
  - family: Lun
    given: Aaron TL
  - family: Smyth
    given: Gordon K
  container-title: In Statistical Analysis of Next Generation Sequence Data. Somnath Datta and Daniel S Nettleton (eds), Springer, New York
  URL: https://link.springer.com/chapter/10.1007%2F978-3-319-07212-8_3
  issued:
    year: 2014
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = showCode, warning = FALSE, message = FALSE)
if (useRAGG) {
  knitr::opts_chunk$set(dev = "ragg_png", dpi = dpi)
}

# knitr::opts_chunk$set(dev.args = list(png = list(type = "cairo")))
knitr::opts_knit$set(progress = FALSE, verbose = FALSE)
```

```{r preparation, message = FALSE}
## Get the number of data sets
nDatasets <- length(ddsList)

## Determine the number of rows and columns in facetted plots 
## (first element = columns, second element = rows)
colRow <- grDevices::n2mfrow(length(ddsList))

## Set height/width of each panel in multi-panel plots
panelSize <- 4

## Define out.width and out.height for the different plot categories
scalefactor <- 96
## barplots, 10x7in
fw1 <- 10
fh1 <- 7
if (useRAGG) {
  ow1 <- paste0(fw1 * scalefactor, "px")
  oh1 <- NULL
} else {
  ow1 <- oh1 <- NULL
}
## multipanel, panelSize * colRow[1] x panelSize * colRow[2]
fw2 <- panelSize * colRow[1]
fh2 <- panelSize * colRow[2]
if (useRAGG) {
  ow2 <- paste0(fw2 * scalefactor, "px")
  oh2 <- NULL
} else {
  ow2 <- oh2 <- NULL
}
## scatter plots etc, 7x5in
fw3 <- 7
fh3 <- 5
if (useRAGG) {
  ow3 <- paste0(fw3 * scalefactor, "px")
  oh3 <- NULL
} else {
  ow3 <- oh3 <- NULL
}

## Initialize plot list
plots <- list()

## Define plot theme
thm <- theme_bw() + 
  theme(axis.text = element_text(size = 15),
        axis.title = element_text(size = 14),
        strip.text = element_text(size = 15))

## ----------------------- Calculate dispersions ----------------------- ##
if (!quiet) {
  message("Calculating dispersions. This can take some time, please be patient.")
}
if (is.finite(maxNForDisp)) {
  msg <- paste0("If there are more than ", maxNForDisp, 
                " samples in a data set, the dispersions are calculated using ",
                maxNForDisp, " randomly selected samples.")
} else {
  msg <- ""
}
obj <- calculateDispersionsddsList(ddsList = ddsList, 
                                   maxNForDisp = maxNForDisp)

## --------------- Calculate between-sample correlations --------------- ##
if (!quiet) {
  message("Calculating sample correlations.")
}
sampleCorrDF <- calculateSampleCorrs(ddsList = obj, 
                                     maxNForCorr = maxNForCorr)

## -------------- Calculate between-feature correlations --------------- ##
if (!quiet) {
  message("Calculating feature correlations.")
}
featureCorrDF <- calculateFeatureCorrs(ddsList = obj, 
                                       maxNForCorr = maxNForCorr)
  
## ----------------- Summarize sample characteristics ------------------ ##
if (!quiet) {
  message("Summarizing calculated values.")
}
sampleDF <- lapply(obj, function(x) {
  data.frame(
    Libsize = colSums(x$dge$counts),
    Fraczero = colMeans(x$dge$counts == 0),
    TMM = x$dge$samples$norm.factors
  ) %>% dplyr::mutate(EffLibsize = Libsize * TMM)
})
ns <- sapply(sampleDF, nrow)
sampleDF <- do.call(rbind, sampleDF) %>% 
  dplyr::mutate(dataset = rep(names(sampleDF), ns))

## ---------------- Summarize feature characteristics ------------------ ##
featureDF <- lapply(obj, function(x) {
  data.frame(
    Tagwise = sqrt(x$dge$tagwise.dispersion),
    Common = sqrt(x$dge$common.dispersion),
    Trend = sqrt(x$dge$trended.dispersion),
    AveLogCPM = x$dge$AveLogCPM,
    AveLogCPMDisp = x$dge$AveLogCPMDisp, 
    average_log2_cpm = apply(edgeR::cpm(x$dge, prior.count = 2, log = TRUE), 1, mean), 
    variance_log2_cpm = apply(edgeR::cpm(x$dge, prior.count = 2, log = TRUE), 1, var),
    Fraczero = rowMeans(x$dge$counts == 0),
    dispGeneEst = rowData(x$dds)$dispGeneEst,
    dispFit = rowData(x$dds)$dispFit,
    dispFinal = rowData(x$dds)$dispersion,
    baseMeanDisp = rowData(x$dds)$baseMeanDisp,
    baseMean = rowData(x$dds)$baseMean
  )
})
ns <- sapply(featureDF, nrow)
featureDF <- do.call(rbind, featureDF) %>% 
  dplyr::mutate(dataset = rep(names(featureDF), ns))

## ----------------- Summarize data set characteristics ---------------- ##
datasetDF <- do.call(rbind, lapply(obj, function(x) {
  data.frame(
    prior_df = paste0("prior.df = ", round(x$dge$prior.df, 2)),
    nVars = nrow(x$dge$counts),
    nSamples = ncol(x$dge$counts)
  )
})) %>% 
  dplyr::mutate(AveLogCPMDisp = 0.8 * max(featureDF$AveLogCPMDisp)) %>%
  dplyr::mutate(Tagwise = 0.9 * max(featureDF$Tagwise)) %>%
  dplyr::mutate(dataset = names(obj))
```

## Description

`r I(description)`

#### Design formulae

The following model formulae were used in the dispersion calculations for the 
different data sets. Note that if a count matrix or data frame was provided in
place of a *DESeqDataSet* for some data set, the corresponding design formula is
set to `~1`, thus assuming that all samples are to be treated as replicates.

```{r, comment = NA}
## Print design information
for (ds in names(ddsList)) {
  cat(ds, ": ", as.character(DESeq2::design(ddsList[[ds]])), "\n")
}
```

## Data set dimensions {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing data set dimensions.")
}
```

These bar plots show the number of samples (columns) and features (rows) in
each data set.

### Number of samples (columns)

```{r nSamples, fig.width = fw1, fig.height = fh1, out.width = ow1, out.height = oh1}
plots[["nSamples"]] <- 
  ggplot(datasetDF, aes(x = dataset, y = nSamples, fill = dataset)) + 
  geom_bar(stat = "identity", alpha = 0.5) + 
  xlab("") + ylab("Number of samples (columns)") + 
  thm + theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5))
plots[["nSamples"]]
```

### Number of features (rows)

```{r nVariables, fig.width = fw1, fig.height = fh1, out.width = ow1, out.height = oh1}
plots[["nVariables"]] <- 
  ggplot(datasetDF, aes(x = dataset, y = nVars, fill = dataset)) + 
  geom_bar(stat = "identity", alpha = 0.5) + 
  xlab("") + ylab("Number of features (rows)") + 
  thm + theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5))
plots[["nVariables"]]
```

## Dispersion/BCV plots {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing dispersion estimates.")
}
```

Disperson/BCV plots show the association between the average abundance and the
dispersion or "biological coefficient of variation" (sqrt(dispersion)), as
calculated by 
[`edgeR`](https://bioconductor.org/packages/release/bioc/html/edgeR.html) 
[@Robinson2010edgeR] and 
[`DESeq2`](http://bioconductor.org/packages/release/bioc/html/DESeq2.html) 
[@Love2014DESeq2]. In the `edgeR` plot, the estimate of the prior degrees of 
freedom is indicated. `r I(msg)`

### edgeR

The black dots represent the tagwise dispersion estimates, the red line the 
common dispersion and the blue curve represents the trended dispersion 
estimates. For further information about the dispersion estimation in `edgeR`,
see @Chen2014Dispersion.

```{r BCVedgeR, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["BCVedgeR"]] <- 
  ggplot(featureDF %>% dplyr::arrange(AveLogCPMDisp), 
         aes(x = AveLogCPMDisp, y = Tagwise)) + 
  geom_point(size = 0.25, alpha = 0.5) + 
  facet_wrap(~dataset, nrow = colRow[2]) + 
  geom_line(aes(y = Trend), color = "blue", size = 1.5) + 
  geom_line(aes(y = Common), color = "red", size = 1.5) +
  geom_text(data = datasetDF, aes(label = prior_df)) + 
  xlab("Average log CPM") + ylab("Biological coefficient of variation") + 
  thm
plots[["BCVedgeR"]]
```

### Pairwise comparisons - edgeR

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "average log CPM and tagwise dispersion", minvalue = 0, maxvalue = 1, permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc2d)`


```{r BCVedgeRTests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = c("AveLogCPMDisp", "Tagwise"), 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

### DESeq2

The black dots are the gene-wise dispersion estimates, the red curve the fitted 
mean-dispersion relationship and the blue circles represent the final dispersion
estimates.For further information about the dispersion estimation in `DESeq2`,
see @Love2014DESeq2.

```{r dispersionDESeq2, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["dispersionDESeq2"]] <- 
  ggplot(featureDF %>% dplyr::arrange(baseMeanDisp), 
         aes(x = baseMeanDisp, y = dispGeneEst)) + 
  geom_point(size = 0.25, alpha = 0.5) + 
  facet_wrap(~dataset, nrow = colRow[2]) + scale_x_log10() + scale_y_log10() +  
  geom_point(aes(y = dispFinal), color = "lightblue", shape = 21) + 
  geom_line(aes(y = dispFit), color = "red", size = 1.5) + 
  xlab("Base mean") + ylab("Dispersion") + 
  thm
plots[["dispersionDESeq2"]]
```

### Pairwise comparisons - DESeq2

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "base mean and genewise dispersion", minvalue = 0, maxvalue = 1, permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc2d)`

```{r dispersionDESeq2Tests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = c("baseMeanDisp", "dispGeneEst"), 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Mean-variance plots {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing mean-variance relationship.")
}
```

These scatter plots show the relation between the empirical mean and variance of
the features. The difference between these mean-variance plots and the 
mean-dispersion plots above is that the plots in this section do not take the 
information about the experimental design and sample grouping into account, but 
simply display the mean and variance of log2(CPM) estimates across all samples, 
calculated using the `cpm` function from 
[`edgeR`](https://bioconductor.org/packages/release/bioc/html/edgeR.html) 
[@Robinson2010edgeR], with a prior count of 2.

### Separate scatter plots

```{r meanVarSepScatter, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["meanVarSepScatter"]] <- 
  ggplot(featureDF, aes(x = average_log2_cpm, y = variance_log2_cpm)) + 
  geom_point(size = 0.75, alpha = 0.5) + 
  facet_wrap(~dataset, nrow = colRow[2]) + 
  xlab("Mean of log2(CPM)") + ylab("Variance of log2(CPM)") + 
  thm
plots[["meanVarSepScatter"]]
```

### Overlaid scatter plots

```{r meanVarOverlaidScatter, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["meanVarOverlaidScatter"]] <- 
  ggplot(featureDF, aes(x = average_log2_cpm, y = variance_log2_cpm, color = dataset)) + 
  geom_point(size = 0.75, alpha = 0.5) +
  xlab("Mean of log2(CPM)") + ylab("Variance of log2(CPM)") + 
  thm
plots[["meanVarOverlaidScatter"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "average and variance of the log CPM", minvalue = 0, maxvalue = 1, permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc2d)` 

```{r meanVarTests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = c("average_log2_cpm", "variance_log2_cpm"), 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Library sizes {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing library sizes.")
}
```

These plots illustrate the distribution of the total read count per sample,
i.e., the column sums of the respective data matrices.

### Separate histograms

```{r libsizeSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["libsizeSepHist"]] <- 
  ggplot(sampleDF, aes(x = Libsize)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Library size") + thm
plots[["libsizeSepHist"]]
```

### Overlaid histograms

```{r libsizeOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeOverlaidHist"]] <- 
  ggplot(sampleDF, aes(x = Libsize, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Library size") + thm
plots[["libsizeOverlaidHist"]]
```

### Density plots

```{r libsizeDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeDensity"]] <- 
  ggplot(sampleDF, aes(x = Libsize, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Library size") + thm
plots[["libsizeDensity"]]
```

### Density plots (filled)

```{r libsizeDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeDensityFilled"]] <- 
  ggplot(sampleDF, aes(x = Libsize, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Library size") + thm
plots[["libsizeDensityFilled"]]
```

### Box plots

```{r libsizeBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeBox"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = Libsize, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Library size") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["libsizeBox"]]
```

### Violin plots

```{r libsizeViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeViolin"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = Libsize, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Library size") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["libsizeViolin"]]
```

### Empirical cumulative distribution function 

```{r libsizeEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeEcdf"]] <- 
  ggplot(sampleDF, aes(x = Libsize, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Library size") + thm
plots[["libsizeEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample", aspect = "library size", minvalue = min(sampleDF$Libsize, na.rm = TRUE), maxvalue = max(sampleDF$Libsize, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`

```{r libsizeTests}
if (calculateStatistics) {
  makeDF(df = sampleDF, column = "Libsize", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## TMM normalization factors {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing TMM normalization factors.")
}
```

The plots below show the distribution of the TMM normalization factors
[@Robinson2010TMM], intended to adjust for differences in RNA composition, as
calculated by 
[`edgeR`](https://bioconductor.org/packages/release/bioc/html/edgeR.html) 
[@Robinson2010edgeR].

### Separate histograms

```{r tmmSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["tmmSepHist"]] <- 
  ggplot(sampleDF, aes(x = TMM)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("TMM normalization factor") + thm
plots[["tmmSepHist"]]
```

### Overlaid histograms

```{r tmmOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmOverlaidHist"]] <- 
  ggplot(sampleDF, aes(x = TMM, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("TMM normalization factor") + thm
plots[["tmmOverlaidHist"]]
```

### Density plots

```{r tmmDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmDensity"]] <- 
  ggplot(sampleDF, aes(x = TMM, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("TMM normalization factor") + thm
plots[["tmmDensity"]]
```

### Density plots (filled) 

```{r tmmDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmDensityFilled"]] <- 
  ggplot(sampleDF, aes(x = TMM, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("TMM normalization factor") + thm
plots[["tmmDensityFilled"]]
```

### Box plots

```{r tmmBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmBox"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = TMM, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("TMM normalization factor") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["tmmBox"]]
```

### Violin plots

```{r tmmViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmViolin"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = TMM, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("TMM normalization factor") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["tmmViolin"]]
```

### Empirical cumulative distribution function 

```{r tmmEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["tmmEcdf"]] <- 
  ggplot(sampleDF, aes(x = TMM, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("TMM normalization factor") + thm
plots[["tmmEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample", aspect = "TMM factor", minvalue = min(sampleDF$TMM, na.rm = TRUE), maxvalue = max(sampleDF$TMM, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`

```{r tmmTests}
if (calculateStatistics) {
  makeDF(df = sampleDF, column = "TMM", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Effective library sizes {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing effective library sizes.")
}
```

These plots show the distribution of the "effective library sizes", defined as
the total count per sample multiplied by the corresponding TMM normalization
factor.

### Separate histograms

```{r effLibsizeSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["effLibsizeSepHist"]] <- 
  ggplot(sampleDF, aes(x = EffLibsize)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Effective library size") + thm
plots[["effLibsizeSepHist"]]
```

### Overlaid histograms

```{r effLibsizeOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeOverlaidHist"]] <- 
  ggplot(sampleDF, aes(x = EffLibsize, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Effective library size") + thm
plots[["effLibsizeOverlaidHist"]]
```

### Density plots

```{r effLibsizeDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeDensity"]] <- 
  ggplot(sampleDF, aes(x = EffLibsize, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Effective library size") + thm
plots[["effLibsizeDensity"]]
```

### Density plots (filled)

```{r effLibsizeDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeDensityFilled"]] <- 
  ggplot(sampleDF, aes(x = EffLibsize, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Effective library size") + thm
plots[["effLibsizeDensityFilled"]]
```

### Box plots

```{r effLibsizeBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeBox"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = EffLibsize, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Effective library size") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["effLibsizeBox"]]
```

### Violin plots

```{r effLibsizeViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeViolin"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = EffLibsize, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Effective library size") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["effLibsizeViolin"]]
```

### Empirical cumulative distribution function 

```{r effLibsizeEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["effLibsizeEcdf"]] <- 
  ggplot(sampleDF, aes(x = EffLibsize, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Effective library size") + thm
plots[["effLibsizeEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample", aspect = "effective library size", minvalue = min(sampleDF$EffLibsize, na.rm = TRUE), maxvalue = max(sampleDF$EffLibsize, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`


```{r effLibsizeTests}
if (calculateStatistics) {
  makeDF(df = sampleDF, column = "EffLibsize", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Expression distributions (average log CPM) {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing expression distributions.")
}
```

The plots in this section show the distribution of average abundance values for
the features. The abundances are log CPM values calculated by `edgeR`.

### Separate histograms

```{r logCPMSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["logCPMSepHist"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Average log CPM") + thm
plots[["logCPMSepHist"]]
```

### Overlaid histograms

```{r logCPMOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMOverlaidHist"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Average log CPM") + thm
plots[["logCPMOverlaidHist"]]
```

### Density plots

```{r logCPMDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMDensity"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Average log CPM") + thm
plots[["logCPMDensity"]]
```

### Density plots (filled) 

```{r logCPMDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMDensityFilled"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Average log CPM") + thm
plots[["logCPMDensityFilled"]]
```

### Box plots

```{r logCPMBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMBox"]] <- 
  ggplot(featureDF, aes(x = dataset, y = AveLogCPM, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Average log CPM") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["logCPMBox"]]
```

### Violin plots

```{r logCPMViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMViolin"]] <- 
  ggplot(featureDF, aes(x = dataset, y = AveLogCPM, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Average log CPM") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["logCPMViolin"]]
```

### Empirical cumulative distribution function 

```{r logCPMEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMEcdf"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Average log CPM") + thm
plots[["logCPMEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "average log CPM", minvalue = min(featureDF$AveLogCPM, na.rm = TRUE), maxvalue = max(featureDF$AveLogCPM, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`


```{r logCPMTests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = "AveLogCPM", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Fraction zeros per sample {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing fraction zeros per sample.")
}
```

These plots show the distribution of the fraction of zeros observed per sample
(column) in the count matrices.

### Separate histograms

```{r fraczeroSampleSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["fraczeroSampleSepHist"]] <- 
  ggplot(sampleDF, aes(x = Fraczero)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Fraction zeros per sample") + thm
plots[["fraczeroSampleSepHist"]]
```

### Overlaid histograms

```{r fraczeroSampleOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleOverlaidHist"]] <- 
  ggplot(sampleDF, aes(x = Fraczero, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Fraction zeros per sample") + thm
plots[["fraczeroSampleOverlaidHist"]]
```

### Density plots

```{r fraczeroSampleDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleDensity"]] <- 
  ggplot(sampleDF, aes(x = Fraczero, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Fraction zeros per sample") + thm
plots[["fraczeroSampleDensity"]]
```

### Density plots (filled) 

```{r fraczeroSampleDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleDensityFilled"]] <- 
  ggplot(sampleDF, aes(x = Fraczero, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Fraction zeros per sample") + thm
plots[["fraczeroSampleDensityFilled"]]
```

### Box plots

```{r fraczeroSampleBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleBox"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = Fraczero, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Fraction zeros per sample") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["fraczeroSampleBox"]]
```

### Violin plots

```{r fraczeroSampleViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleViolin"]] <- 
  ggplot(sampleDF, aes(x = dataset, y = Fraczero, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Fraction zeros per sample") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["fraczeroSampleViolin"]]
```

### Empirical cumulative distribution function 

```{r fraczeroSampleEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroSampleEcdf"]] <- 
  ggplot(sampleDF, aes(x = Fraczero, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Fraction zeros per sample") + thm
plots[["fraczeroSampleEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample", aspect = "fraction zeros", minvalue = min(sampleDF$Fraczero, na.rm = TRUE), maxvalue = max(sampleDF$Fraczero, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`
 

```{r fraczeroSampleTests}
if (calculateStatistics) {
  makeDF(df = sampleDF, column = "Fraczero", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Fraction zeros per feature {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing fraction zeros per feature.")
}
```

These plots illustrate the distribution of the fraction of zeros observed per
feature (row) in the count matrices.

### Separate histograms

```{r fraczeroFeatureSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["fraczeroFeatureSepHist"]] <- 
  ggplot(featureDF, aes(x = Fraczero)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Fraction zeros per feature") + thm
plots[["fraczeroFeatureSepHist"]]
```

### Overlaid histograms

```{r fraczeroFeatureOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureOverlaidHist"]] <- 
  ggplot(featureDF, aes(x = Fraczero, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Fraction zeros per feature") + thm
plots[["fraczeroFeatureOverlaidHist"]]
```

### Density plots

```{r fraczeroFeatureDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureDensity"]] <- 
  ggplot(featureDF, aes(x = Fraczero, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Fraction zeros per feature") + thm
plots[["fraczeroFeatureDensity"]]
```

### Density plots (filled) 

```{r fraczeroFeatureDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureDensityFilled"]] <- 
  ggplot(featureDF, aes(x = Fraczero, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Fraction zeros per feature") + thm
plots[["fraczeroFeatureDensityFilled"]]
```

### Box plots

```{r fraczeroFeatureBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureBox"]] <- 
  ggplot(featureDF, aes(x = dataset, y = Fraczero, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Fraction zeros per feature") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["fraczeroFeatureBox"]]
```

### Violin plots

```{r fraczeroFeatureViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureViolin"]] <- 
  ggplot(featureDF, aes(x = dataset, y = Fraczero, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Fraction zeros per feature") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["fraczeroFeatureViolin"]]
```

### Empirical cumulative distribution function 

```{r fraczeroFeatureEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["fraczeroFeatureEcdf"]] <- 
  ggplot(featureDF, aes(x = Fraczero, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Fraction zeros per feature") + thm
plots[["fraczeroFeatureEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "fraction zeros", minvalue = min(featureDF$Fraczero, na.rm = TRUE), maxvalue = max(featureDF$Fraczero, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`


```{r fraczeroFeatureTests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = "Fraczero", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Sample-sample correlations {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing sample-sample correlations.")
}
```

The plots below show the distribution of Spearman correlation coefficients for
pairs of samples, calculated from the log(CPM) values obtained via the `cpm`
function from `edgeR`, with a prior.count of 2. If there are more than 
`r maxNForCorr` samples in a data set, the pairwise correlations between 
`r maxNForCorr` randomly selected samples are shown.

### Separate histograms

```{r sampleCorrSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["sampleCorrSepHist"]] <- 
  ggplot(sampleCorrDF, aes(x = Correlation)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Sample-sample correlation") + thm
plots[["sampleCorrSepHist"]]
```

### Overlaid histograms

```{r sampleCorrOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrOverlaidHist"]] <- 
  ggplot(sampleCorrDF, aes(x = Correlation, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Sample-sample correlation") + thm
plots[["sampleCorrOverlaidHist"]]
```

### Density plots

```{r sampleCorrDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrDensity"]] <- 
  ggplot(sampleCorrDF, aes(x = Correlation, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Sample-sample correlation") + thm
plots[["sampleCorrDensity"]]
```

### Density plots (filled)

```{r sampleCorrDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrDensityFilled"]] <- 
  ggplot(sampleCorrDF, aes(x = Correlation, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Sample-sample correlation") + thm
plots[["sampleCorrDensityFilled"]]
```

### Box plots

```{r sampleCorrBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrBox"]] <- 
  ggplot(sampleCorrDF, aes(x = dataset, y = Correlation, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Sample-sample correlation") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["sampleCorrBox"]]
```

### Violin plots

```{r sampleCorrViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrViolin"]] <- 
  ggplot(sampleCorrDF, aes(x = dataset, y = Correlation, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Sample-sample correlation") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["sampleCorrViolin"]]
```

### Empirical cumulative distribution function 

```{r sampleCorrEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["sampleCorrEcdf"]] <- 
  ggplot(sampleCorrDF, aes(x = Correlation, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Sample-sample correlation") + thm
plots[["sampleCorrEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample pair", aspect = "Spearman correlation", minvalue = min(sampleCorrDF$Correlation, na.rm = TRUE), maxvalue = max(sampleCorrDF$Correlation, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`
 

```{r sampleCorrTests}
if (calculateStatistics) {
  makeDF(df = sampleCorrDF, column = "Correlation", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Feature-feature correlations {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing feature-feature correlations.")
}
```

These plots illustrate the distribution of Spearman correlation coefficients for
pairs of features, calculated from the log(CPM) values obtained via the `cpm`
function from `edgeR`, with a prior.count of 2. Only non-constant features are
considered, and if there are more than `r maxNForCorr` such features in a
data set, the pairwise correlations between `r maxNForCorr` randomly selected
features are shown.

### Separate histograms

```{r featureCorrSepHist, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["featureCorrSepHist"]] <- 
  ggplot(featureCorrDF, aes(x = Correlation)) + geom_histogram(bins = 30) + 
  facet_wrap(~dataset, nrow = colRow[2]) +
  xlab("Feature-feature correlation") + thm
plots[["featureCorrSepHist"]]
```

### Overlaid histograms

```{r featureCorrOverlaidHist, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrOverlaidHist"]] <- 
  ggplot(featureCorrDF, aes(x = Correlation, group = dataset, fill = dataset)) + 
  geom_histogram(bins = 30, alpha = 0.5, position = "identity") +
  xlab("Feature-feature correlation") + thm
plots[["featureCorrOverlaidHist"]]
```

### Density plots

```{r featureCorrDensity, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrDensity"]] <- 
  ggplot(featureCorrDF, aes(x = Correlation, group = dataset, color = dataset)) + 
  geom_line(alpha = 0.5, size = 1.5, stat = "density") +
  xlab("Feature-feature correlation") + thm
plots[["featureCorrDensity"]]
```

### Density plots (filled)

```{r featureCorrDensityFilled, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrDensityFilled"]] <- 
  ggplot(featureCorrDF, aes(x = Correlation, group = dataset, fill = dataset)) + 
  geom_density(alpha = 0.5) +
  xlab("Feature-feature correlation") + thm
plots[["featureCorrDensityFilled"]]
```

### Box plots

```{r featureCorrBox, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrBox"]] <- 
  ggplot(featureCorrDF, aes(x = dataset, y = Correlation, color = dataset)) + 
  geom_boxplot(outlier.size = -1, alpha = 0.5) + 
  geom_point(position = position_jitter(width = 0.2), size = 2, alpha = 0.5) + 
  ylab("Feature-feature correlation") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["featureCorrBox"]]
```

### Violin plots

```{r featureCorrViolin, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrViolin"]] <- 
  ggplot(featureCorrDF, aes(x = dataset, y = Correlation, fill = dataset)) + 
  geom_violin(alpha = 0.5) + 
  ylab("Feature-feature correlation") + xlab("") + thm + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
plots[["featureCorrViolin"]]
```

### Empirical cumulative distribution function 

```{r featureCorrEcdf, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["featureCorrEcdf"]] <- 
  ggplot(featureCorrDF, aes(x = Correlation, group = dataset, color = dataset)) + 
  stat_ecdf(size = 1.5, alpha = 0.5) + ylab("Cumulative proportion") + 
  xlab("Feature-feature correlation") + thm
plots[["featureCorrEcdf"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature pair", aspect = "Spearman correlation", minvalue = min(featureCorrDF$Correlation, na.rm = TRUE), maxvalue = max(featureCorrDF$Correlation, na.rm = TRUE), permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc)`
 

```{r featureCorrTests}
if (calculateStatistics) {
  makeDF(df = featureCorrDF, column = "Correlation", 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Library size vs fraction zeros {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing library size vs fraction zeros relationship.")
}
```

These scatter plots show the association between the total count (column sums)
and the fraction of zeros observed per sample.

### Separate scatter plots

```{r libsizeFraczeroSepScatter, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["libsizeFraczeroSepScatter"]] <- 
  ggplot(sampleDF, aes(x = Libsize, y = Fraczero)) + 
  geom_point(size = 1, alpha = 0.5) + 
  facet_wrap(~dataset, nrow = colRow[2]) + 
  xlab("Library size") + ylab("Fraction zeros") + thm
plots[["libsizeFraczeroSepScatter"]]
```

### Overlaid scatter plots

```{r libsizeFraczeroOverlaidScatter, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["libsizeFraczeroOverlaidScatter"]] <- 
  ggplot(sampleDF, aes(x = Libsize, y = Fraczero, color = dataset)) + 
  geom_point(size = 1, alpha = 0.5) + 
  geom_line(stat = "smooth", method = "loess", size = 1, alpha = 0.5) + 
  xlab("Library size") + ylab("Fraction zeros") + thm
plots[["libsizeFraczeroOverlaidScatter"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "sample", aspect = "library size and fraction of zeros", minvalue = 0, maxvalue = 1, permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc2d)` 

```{r libsizeFraczeroTests}
if (calculateStatistics) {
  makeDF(df = sampleDF, column = c("Libsize", "Fraczero"), 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```

## Mean expression vs fraction zeros {.tabset .tabset-pills}

```{r, include = FALSE}
if (!quiet) {
  message("Processing expression vs fraction zeros relationship.")
}
```

These scatter plots show the association between the average abundance and the 
fraction of zeros observed per feature. The abundance is defined as the
log(CPM) values as calculated by `edgeR`.

### Separate scatter plots

```{r logCPMFraczeroSepScatter, fig.width = fw2, fig.height = fh2, out.width = ow2, out.height = oh2}
plots[["logCPMFraczeroSepScatter"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, y = Fraczero)) + 
  geom_point(size = 0.75, alpha = 0.5) + 
  facet_wrap(~dataset, nrow = colRow[2]) + 
  xlab("Average log CPM") + ylab("Fraction zeros") + thm
plots[["logCPMFraczeroSepScatter"]]
```

### Overlaid scatter plots

```{r logCPMFraczeroOverlaidScatter, fig.width = fw3, fig.height = fh3, out.width = ow3, out.height = oh3}
plots[["logCPMFraczeroOverlaidScatter"]] <- 
  ggplot(featureDF, aes(x = AveLogCPM, y = Fraczero, color = dataset)) + 
  geom_point(size = 0.75, alpha = 0.5) + 
  xlab("Average log CPM") + ylab("Fraction zeros") + thm
plots[["logCPMFraczeroOverlaidScatter"]]
```

### Pairwise comparisons

`r I(defineTableDesc(calculateStatistics = calculateStatistics, subsampleSize = subsampleSize, kfrac = kfrac, kmin = kmin, obstype = "feature", aspect = "average log CPM and fraction of zeros", minvalue = 0, maxvalue = 1, permutationPvalues = permutationPvalues, nPermutations = nPermutations, nDatasets = nDatasets)$tabledesc2d)` 

```{r logCPMFraczeroTests}
if (calculateStatistics) {
  makeDF(df = featureDF, column = c("AveLogCPM", "Fraczero"), 
         permutationPvalues = permutationPvalues, nPermutations = nPermutations, 
         subsampleSize = subsampleSize, kmin = kmin, kfrac = kfrac)
}
```


```{r, include = FALSE}
if (savePlots) {
  if (!quiet) {
    message("Saving plots.")
  }
  saveRDS(plots, 
          file = paste0(outputDir, "/",
                        tools::file_path_sans_ext(basename(outputFile)), "_ggplots.rds"))
}
```

## Session info

```{r}
sessionInfo()
```

## References

---
title: "countsimQC - Comparing characteristic features across count data sets"
author: "Charlotte Soneson"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{countsimQC User Guide}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(echo = TRUE,
                      crop = NA)
```

## Introduction

The `countsimQC` package provides a simple way to compare the characteristic 
features of a collection of (e.g., RNA-seq) count data sets. An important
application is in situations where a synthetic count data set has been generated
using a real count data set as an underlying source of parameters, in which case
it is often important to verify that the final synthetic data captures the main 
features of the original data set. However, the package can be used to create a 
visual overview of any collection of one or more count data sets.

## Data preparation

In this vignette we will show how to generate a comparative report from a 
collection of two simulated data sets and the original, underlying real data 
set. First, we load the object containing the three data sets. The object is a 
named list, where each element is a `DESeqDataSet` object, containing the count 
matrix, a sample information data frame and a model formula (necessary to 
calculate dispersions). For more information about the `DESeqDataSet` class, 
please see the
[`DESeq2`](http://bioconductor.org/packages/release/bioc/html/DESeq2.html)
Bioconductor package. For speed reasons, we use only a subset of the features 
in each data set for the following calculations.

```{r, warning = FALSE}
suppressPackageStartupMessages({
  library(countsimQC)
  library(DESeq2)
})

data(countsimExample)
countsimExample
countsimExample <- lapply(countsimExample, function(cse) {
  cse[seq_len(1500), ]
})
```

## Report generation

Next, we generate the report using the `countsimQCReport()` function. Depending
on the level of detail and the type of information that are required for the
final report, this function can be run in different "modes":

- by setting `calculateStatistics = FALSE`, only plots will be generated. This
is the fastest way of running `countsimQCReport()`, and in many cases generates
enough information for the user to make a visual evaluation of the count data
set(s).
- by setting `calculateStatistics = TRUE` and `permutationPvalues = FALSE`, some
quantitative pairwise comparisons between data sets will be performed. In
particular, the Kolmogorov-Smirnov test and the Wald-Wolfowitz runs test will be
used to compare distributions, and additional statistics will be calculated to
evaluate how similar the evaluated aspects are between pairs of data sets.
- by setting `calculateStatistics = TRUE` and `permutationPvalues = TRUE` (and
giving the requested number of permutations via the `nPermutations` argument),
permutation of data set labels will be used to evaluate the significance of the
statistics calculated in the previous point. Naturally, this increases the run
time of the analysis considerably.

Here, for the sake of speed, we calculate statistics for a small subset of the
observations (`subsampleSize = 25`) and refrain from calculating permutation
p-values. 

```{r, results = "hide", warning = FALSE, message = FALSE}
tempDir <- tempdir()
countsimQCReport(ddsList = countsimExample, outputFile = "countsim_report.html",
                 outputDir = tempDir, outputFormat = "html_document", 
                 showCode = FALSE, forceOverwrite = TRUE,
                 savePlots = TRUE, description = "This is my test report.", 
                 maxNForCorr = 25, maxNForDisp = Inf, 
                 calculateStatistics = TRUE, subsampleSize = 25,
                 kfrac = 0.01, kmin = 5, 
                 permutationPvalues = FALSE, nPermutations = NULL)
```

The `countsimQCReport()` function can generate either an HTML file (by setting
`outputFormat = "html_document"` or `outputFormat = NULL`) or a pdf file (by
setting `outputFormat = "pdf_document"`). The `description` argument can be used
to provide a more extensive description of the data set(s) that are included in
the report.

## Generation of individual figures

If the argument `savePlots` is set to TRUE, an .rds file containing the 
individual ggplot objects will be generated. These objects can be used to
perform fine-tuning of the visualizations if desired. Note, however, that the
.rds file can become large if the number of data sets is large, or if the
individual data sets have many samples or features. The convenience function 
`generateIndividualPlots()` can be used to quickly generate individual figures
for all plots included in the report, using a variety of devices. For example,
to generate each plot in pdf format:

```{r, warning = FALSE}
ggplots <- readRDS(file.path(tempDir, "countsim_report_ggplots.rds"))
if (!dir.exists(file.path(tempDir, "figures"))) {
  dir.create(file.path(tempDir, "figures"))
}
generateIndividualPlots(ggplots, device = "pdf", nDatasets = 3, 
                        outputDir = file.path(tempDir, "figures"))
```

## Input data format

In the example above, all data sets were provided as `DESeqDataSet` objects. The
advantage of this is that it allows the specification of the experimental
design, which is used in the dispersion calculations. `countsimQC` also allows a
data set to be provided as either a `data.frame` or a `matrix`. However, in
these situations, it will be assumed that all samples are replicates (i.e., a
design `~1`). An example is provided in the `countsimExample_dfmat` data set,
provided with the package.

```{r}
data(countsimExample_dfmat)
names(countsimExample_dfmat)
lapply(countsimExample_dfmat, class)
```

```{r, results = "hide", warning = FALSE, eval = FALSE}
tempDir <- tempdir()
countsimQCReport(ddsList = countsimExample_dfmat, 
                 outputFile = "countsim_report_dfmat.html",
                 outputDir = tempDir, outputFormat = "html_document", 
                 showCode = FALSE, forceOverwrite = TRUE,
                 savePlots = TRUE, description = "This is my test report.", 
                 maxNForCorr = 25, maxNForDisp = Inf, 
                 calculateStatistics = TRUE, subsampleSize = 25,
                 kfrac = 0.01, kmin = 5, 
                 permutationPvalues = FALSE, nPermutations = NULL)
```

## Session info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateDispersionsddsList.R
\name{calculateDispersionsddsList}
\alias{calculateDispersionsddsList}
\title{Calculate dispersions}
\usage{
calculateDispersionsddsList(ddsList, maxNForDisp)
}
\arguments{
\item{ddsList}{A list of DESeqDataSets}

\item{maxNForDisp}{If any data set contains more than \code{maxNForDisp}
samples, \code{maxNForDisp} of them will be randomly sampled before the
dispersions are calculated, in order to speed up calculations}
}
\value{
A list of the same length as the input list. Each element in the list
  is itself a list, containing a DGEList and a DESeqDataSet with calculated
  dispersions.
}
\description{
Calculate the dispersions for each data set in a list of DESeqDataSets, using
both edgeR and DESeq2.
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDF.R
\name{defineTableDesc}
\alias{defineTableDesc}
\title{Define table descriptions}
\usage{
defineTableDesc(
  calculateStatistics,
  subsampleSize,
  kfrac,
  kmin,
  obstype,
  aspect,
  minvalue,
  maxvalue,
  permutationPvalues,
  nPermutations,
  nDatasets
)
}
\arguments{
\item{calculateStatistics}{Whether or not statistics and p-values are
calculated}

\item{subsampleSize}{The number of observations for which certain
(time-consuming) statistics will be calculated}

\item{kmin, kfrac}{For statistics that require the extraction of k nearest
neighbors of a given point, the number of neighbors will be max(kmin, kfrac
* nrow(df))}

\item{obstype}{The type of observation (e.g., sample, feature, sample pair)}

\item{aspect}{The name of the aspect of interest}

\item{minvalue, maxvalue}{The minimal and maximal value of the aspect of
interest, used for scaling of the x axis when calculating the area between
the eCDFs}

\item{permutationPvalues}{Whether or not to calculate p-values of statistics
via permutation}

\item{nPermutations}{The number of permutations (only used if
permutationPvalues = TRUE)}

\item{nDatasets}{The number of data sets that are being compared}
}
\value{
A list with two text strings in markdown format: one for tables based
  on a single data column, and one for tables based on two data columns
}
\description{
Generate the text that describes the content of the tables generated by
\code{makeDF}.
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateIndividualPlots.R
\name{generateIndividualPlots}
\alias{generateIndividualPlots}
\title{Generate individual plots from countsimQCReport output}
\usage{
generateIndividualPlots(
  ggplotsRds,
  device = "png",
  outputDir = "./",
  nDatasets = 2
)
}
\arguments{
\item{ggplotsRds}{The path to a .rds file generated by
\code{countsimQCReport} by setting \code{savePlots = TRUE}, or the list of
plots stored in this file.}

\item{device}{One of "eps", "ps", "tex" (pictex), "pdf", "jpeg", "tiff",
"png", "bmp", "svg" or "wmf" (windows only) (will be provided to the
\code{ggsave} function from the \code{ggplot2} package).}

\item{outputDir}{The output directory where the plots should be generated.}

\item{nDatasets}{The number of data sets that are compared in the figures.
This is needed to set the size of the plots correctly.}
}
\value{
Nothing is returned, but plots are generated in the designated
  output directory.
}
\description{
Generate separate plots for all evaluation criteria using the collection of
ggplot objects that can be saved when generating a countsimQC report (by
setting \code{savePlots = TRUE}).
}
\examples{
## Load example data
data(countsimExample)
\dontrun{
## Generate report
countsimQCReport(countsimExample, outputDir = "./",
                 outputFile = "example.html", savePlots = TRUE)
## Generate individual plots
generateIndividualPlots("example_ggplots.rds", nDatasets = 3)
}

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countsimExample.R
\docType{data}
\name{countsimExample}
\alias{countsimExample}
\title{Example list with three count data sets}
\format{
A named list with three elements, each corresponding to a (real or
  simulated) count data set.
}
\usage{
data("countsimExample")
}
\value{
A named list with three elements, each corresponding to a (real or
  simulated) count data set.
}
\description{
A named list with three elements, each corresponding to a (real or simulated)
count data set.
}
\details{
The \code{Original} data set represents a subset of 10,000 genes and 11 cells
from the GSE74596 single-cell RNA-seq data set, obtained from the conquer
repository (http://imlspenticton.uzh.ch:3838/conquer/). The \code{Sim1} and
\code{Sim2} data sets similarly represent subsets of scRNA-seq data sets
simulated with two different simulation methods, using the real GSE74596 data
set as the basis for parameter estimation. Each data set is represented as a
\code{DESeqDataSet} object.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDF.R
\name{makeDF}
\alias{makeDF}
\title{Construct data frame with pairwise statistics}
\usage{
makeDF(
  df,
  column,
  permutationPvalues,
  nPermutations,
  subsampleSize,
  kmin,
  kfrac
)
}
\arguments{
\item{df}{The input data frame. Must contain at least a column named
'dataset' and an additional column with values}

\item{column}{The name of the column(s) of \code{df} to be used as the basis
for the comparison}

\item{permutationPvalues}{Whether or not to calculate p-values of statistics
via permutation}

\item{nPermutations}{The number of permutations (only used if
permutationPvalues = TRUE)}

\item{subsampleSize}{The number of observations for which certain
(time-consuming) statistics will be calculated. The observations will be
selected randomly among the rows of \code{df}}

\item{kmin, kfrac}{For statistics that require the extraction of k nearest
neighbors of a given point, the number of neighbors will be max(kmin, kfrac
* nrow(df))}
}
\value{
A data table with statistics and p-values for pairwise comparisons of
  data sets, based on the provided \code{column}
}
\description{
Construct a data frame containing statistics and p-values for pairwise
comparison of data sets.
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countsimQCReport.R
\name{countsimQCReport}
\alias{countsimQCReport}
\title{Generate countsimQC report}
\usage{
countsimQCReport(
  ddsList,
  outputFile,
  outputDir = "./",
  outputFormat = NULL,
  showCode = FALSE,
  rmdTemplate = NULL,
  forceOverwrite = FALSE,
  savePlots = FALSE,
  description = NULL,
  maxNForCorr = 500,
  maxNForDisp = Inf,
  calculateStatistics = TRUE,
  subsampleSize = 500,
  kfrac = 0.01,
  kmin = 5,
  permutationPvalues = FALSE,
  nPermutations = NULL,
  knitrProgress = FALSE,
  quiet = FALSE,
  ignorePandoc = FALSE,
  useRAGG = FALSE,
  dpi = 96,
  ...
)
}
\arguments{
\item{ddsList}{Named list of \code{DESeqDataSets} or count matrices to
compare. See the \code{DESeq2} Bioconductor package
(http://bioconductor.org/packages/release/bioc/html/DESeq2.html) for more
information about the \code{DESeqDataSet} class. Each \code{DESeqDataSet}
object in the list should contain a count matrix, a data frame with sample
information and a design formula. The sample information and design formula
will be used to calculate dispersions appropriately. If count matrices are
provided, it is assumed that all columns represent replicate samples, and
the design formula ~1 will be used.}

\item{outputFile}{The file name of the final report. The extension must match
the selected \code{outputFormat} (i.e., either .html or .pdf).}

\item{outputDir}{The directory where the final report should be saved.}

\item{outputFormat}{The output format of the report. If set to NULL or
"html_document", an html report will be generated. If set to
"pdf_document", a pdf report will be generated.}

\item{showCode}{Whether or not to include the code in the final report.}

\item{rmdTemplate}{The Rmarkdown (.Rmd) file that will be used as the
template for generating the report. If set to NULL (default), the template
provided with the countsimQC package will be used. See Details for more
information.}

\item{forceOverwrite}{Whether to force overwrite existing output files when
saving the generated report and figures.}

\item{savePlots}{Whether to save the ggplot objects for all the output
figures, to allow additional fine-tuning and generation of individual
plots. Note that the resulting file can be quite large, especially when
many and/or large data sets are compared.}

\item{description}{A string (of arbitrary length) describing the content of
the generated report. This will be included in the beginning of the report.
If set to NULL, a default description listing the number and names of the
included data sets will be used.}

\item{maxNForCorr}{The maximal number of samples (features) for which
pairwise correlation coefficients will be calculated. If the number of
samples (features) exceeds this number, they will be randomly subsampled.}

\item{maxNForDisp}{The maximal number of samples that will be used to
estimate dispersions. By default, all samples are used. This can be lowered
to speed up calculations (and obtain approximate results) for large data
sets.}

\item{calculateStatistics}{Whether to calculate quantitative pairwise
statistics for comparing data sets in addition to generating the plots.}

\item{subsampleSize}{The number of randomly selected observations (samples,
features or pairs of samples or features) for which certain
(time-consuming) statistics will be calculated. Only used if
\code{calculateStatistics} = TRUE.}

\item{kmin, kfrac}{For statistics that require the extraction of the k nearest
neighbors of a given point, the number of neighbors will be max(kmin, kfrac
* nrow(df))}

\item{permutationPvalues}{Whether to calculate permutation p-values for
selected pairwise data set comparison statistics.}

\item{nPermutations}{The number of permutations to perform when calculating
permutation p-values for data set comparison statistics. Only used if
\code{permutationPvalues} = TRUE.}

\item{knitrProgress}{Whether to show the progress bar when the report is
generated.}

\item{quiet}{Whether to suppress warnings and progress messages when the
report is generated.}

\item{ignorePandoc}{Determines what to do if pandoc or pandoc-citeproc is
missing (if Sys.which("pandoc") or Sys.which("pandoc-citeproc") is ""). If
ignorePandoc is TRUE, only a warning is given. The figures will be
generated, but not the final report. If ignorePandoc is FALSE (default),
the execution stops immediately.}

\item{useRAGG}{Logical scalar, indicating whether to use ragg_png as the
graphics device in the report rather than the default png.}

\item{dpi}{Numeric scalar, setting the dpi of the generated plots. Only
used if \code{useRAGG} is \code{TRUE}.}

\item{...}{Other arguments that will be passed to \code{rmarkdown::render}.}
}
\value{
No value is returned, but a report is generated in the
  \code{outputDir} directory.
}
\description{
Generate a report comparing a range of characteristics across a collection of
one or more count data sets.
}
\details{
When the function is called, the template file (specified by
  \code{rmdTemplate}) will be copied into the output folder, and
  \code{rmarkdown::render} will be called to generate the final report. If
  there is already a .Rmd file with the same name in the output folder, the
  function will raise an error and stop, to avoid overwriting the existing
  file. The reason for this behaviour is that the copied template in the
  output folder will be deleted once the report is generated.
}
\examples{
## Load example data
data(countsimExample)
\dontrun{
## Generate report
countsimQCReport(countsimExample, outputDir = "./",
                 outputFile = "example.html")
}

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCorrelations.R
\name{calculateSampleCorrs}
\alias{calculateSampleCorrs}
\title{Calculate Spearman correlation between sample pairs}
\usage{
calculateSampleCorrs(ddsList, maxNForCorr)
}
\arguments{
\item{ddsList}{List of lists, with one element per data set. Each element is
a list containing a DGEList and a DESeqDataSet, with calculated
dispersions (e.g., output from \code{\link{calculateDispersionsddsList}}).}

\item{maxNForCorr}{Maximal number of samples to use for correlation
calculation. If the number of samples in a data set exceeds
\code{maxNForCorr}, \code{maxNForCorr} samples will be randomly selected
for calculation of correlations.}
}
\value{
A data frame with pairwise sample correlations for each data set
}
\description{
Calculate Spearman correlation between sample pairs
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStats.R
\name{defaultStats}
\alias{defaultStats}
\title{Return a vector of NA scores}
\usage{
defaultStats(n, withP = FALSE)
}
\arguments{
\item{n}{Number of columns to use for the comparison}

\item{withP}{Whether or not to include p-value columns}
}
\value{
A vector with NA values for all applicable statistics
}
\description{
Return a vector of NA scores
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateStats.R
\name{calculateStats}
\alias{calculateStats}
\title{Calculate statistics for pairwise comparison of data sets}
\usage{
calculateStats(
  df,
  ds1,
  ds2,
  column,
  subsampleSize,
  permute = FALSE,
  kmin,
  kfrac,
  xmin,
  xmax
)
}
\arguments{
\item{df}{The input data frame. Must contain at least a column named
'dataset' and an additional column with values to use as the basis for the
comparison.}

\item{ds1, ds2}{The names of the two data sets to be compared.}

\item{column}{The name of the column(s) of \code{df} to be used as the basis
for the comparison.}

\item{subsampleSize}{The number of observations for which certain
time-consuming statistics will be calculated. The observations will be
selected randomly among the rows of \code{df}.}

\item{permute}{Whether to permute the dataset column of \code{df} before
calculating the statistics.}

\item{kmin, kfrac}{For statistics that require the extraction of k nearest
neighbors of a given point, the number of neighbors will be max(kmin, kfrac
* nrow(df)).}

\item{xmin, xmax}{Smallest and largest value of \code{column}, used to
normalize the x-axis when calculating the area between the eCDFs.}
}
\value{
A vector with statistics and p-values
}
\description{
Calculate a range of statistics and p-values for comparison of two data sets.
}
\author{
Charlotte Soneson
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countsimQC-pkg.R
\docType{package}
\name{countsimQC-package}
\alias{countsimQC}
\alias{countsimQC-package}
\title{countsimQC}
\description{
countsimQC
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/csoneson/countsimQC}
  \item Report bugs at \url{https://github.com/csoneson/countsimQC/issues}
}

}
\author{
\strong{Maintainer}: Charlotte Soneson \email{charlottesoneson@gmail.com} (\href{https://orcid.org/0000-0003-3833-2169}{ORCID})

}
\keyword{internal}
